package net.ae7q.memory;

//==================================================================================================
/** Ft817calibration implements the calibration data for an FT-817.
 *
 *  @version    2006.04.09
 *  @author     Copyright © 2006 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
class Ft817calibration extends Ft___Calibration {

    //==============================================================================================
    /** Field describes all the radio's calibration settings.
     */
    public enum Field implements FtField, FtEnumPrefix {
        _HF1_RxG,               // ( 1) // 0x00
        _HF2_RxG,               // ( 2) // 0x01
        _HF3_RxG,               // ( 3) // 0x02
        _50m_RxG,               // ( 4) // 0x03
        _VHF_RxG,               // ( 5) // 0x04
        _UHF_RxG,               // ( 6) // 0x05
        _SSB_S9,                // ( 7) // 0x06
        _SSB_FS,                // ( 8) // 0x07
        _FN_S1,                 // ( 9) // 0x08
        _FM_FS,                 // (10) // 0x09
        _Disc_L,                // (11) // 0x0A
        _Disc_H,                // (12) // 0x0B
        _FM_Th1,                // (13) // 0x0C
        _FM_Th2,                // (14) // 0x0D
        _FM_Ti1,                // (15) // 0x0E
        _FM_Ti2,                // (16) // 0x0F
        _VCC,                   // (17) // 0x10
        _HF1_Ic,                // (18) // 0x11
        _HF2_Ic,                // (19) // 0x12
        _HF3_Ic,                // (20) // 0x13
        _50m_Ic,                // (21) // 0x14
        _VHF_Ic,                // (22) // 0x15
        _UHF_Ic,                // (23) // 0x16
        _HF1_Hi,                // (24) // 0x17
        _HF1_L3,                // (25) // 0x18
        _HF1_L2,                // (26) // 0x19
        _HF1_L1,                // (27) // 0x1A
        _HF2_Hi,                // (28) // 0x1B
        _HF2_L3,                // (29) // 0x1C
        _HF2_L2,                // (30) // 0x1D
        _HF2_L1,                // (31) // 0x1E
        _HF3_Hi,                // (32) // 0x1F
        _HF3_L3,                // (33) // 0x20
        _HF3_L2,                // (34) // 0x21
        _HF3_L1,                // (35) // 0x22
        _50m_Hi,                // (36) // 0x23
        _50m_L3,                // (37) // 0x24
        _50m_L2,                // (38) // 0x25
        _50m_L1,                // (39) // 0x26
        _VHF_Hi,                // (40) // 0x27
        _VHF_L3,                // (41) // 0x28
        _VHF_L2,                // (42) // 0x29
        _VHF_L1,                // (43) // 0x2A
        _UHF_Hi,                // (44) // 0x2B
        _UHF_L3,                // (45) // 0x2C
        _UHF_L2,                // (46) // 0x2D
        _UHF_L1,                // (47) // 0x2E
        _HF1_TxG,               // (48) // 0x2F
        _HF2_TxG,               // (49) // 0x30
        _HF3_TxG,               // (50) // 0x31
        _50m_TxG,               // (51) // 0x32
        _VHF_TxG,               // (52) // 0x33
        _UHF_TxG,               // (53) // 0x34
        _HF1_Pom,               // (54) // 0x35
        _HF2_Pom,               // (55) // 0x36
        _HF3_Pom,               // (56) // 0x37
        _50m_Pom,               // (57) // 0x38
        _VHF_Pom,               // (58) // 0x39
        _UHF_Pom,               // (59) // 0x3A
        _ALC1_M,                // (60) // 0x3B
        _ALC_M,                 // (61) // 0x3C
        _HF1_Rv,                // (62) // 0x3D
        _HF2_Rv,                // (63) // 0x3E
        _HF3_Rv,                // (64) // 0x3F
        _50m_Rv,                // (65) // 0x40
        _VHF_Rv,                // (66) // 0x41
        _UHF_Rv,                // (67) // 0x42
        _CW_Car,                // (68) // 0x43
        _AM_Car,                // (69) // 0x44
        _Dev_W,                 // (70) // 0x45
        _Dev_N,                 // (71) // 0x46
        _M_Mtr,                 // (72) // 0x47
        _CTCSS,                 // (73) // 0x48
        _DCS,                   // (74) // 0x49
        _LSB_CP ( true ),       // (75) // 0x4A
        _USB_CP ( true ),       // (76) // 0x4B
        ;
        private final int     min, max;

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         */
        private Field(){
            this( false );
        } // Ft817calibration.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  signed                      whether the value is signed.
         */
        private Field( boolean signed ){
            min = signed ? Byte.MIN_VALUE : 0;
            max = Byte.MAX_VALUE - Byte.MIN_VALUE + min;
        } // Ft817calibration.Field constructor

        //------------------------------------------------------------------------------------------
        /** get retrieves the field from a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @return Object                      - the field.
         */
        public Object get( byte[] buffer ){
            return min < 0 ? buffer[ ordinal() ] : 0xFF & buffer[ ordinal() ];
        } // Ft817calibration.Field.get

        //------------------------------------------------------------------------------------------
        /** put places a field into a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @param  object                      the field.
         */
        public void put( byte[] buffer, Object object ){
            buffer[ ordinal() ] = (byte) FtRemote.putValue( 0, 1, min, max, null, object );
        } // Ft817calibration.Field.put

        //------------------------------------------------------------------------------------------
        /** type returns the Type of the field.
         *
         *  @return Type                        - the Type of field.
         */
        public Type type(){
            return min < 0 ? Type.SIGNED : Type.OTHER;
        } // Ft817calibration.Field.type

    } // Ft817calibration.Field enum

    static private final long    serialVersionUID = 2006040900L;
    static public  final int[]   addresses        = { 0x0007, 0x0053 },
                                 limits           = { 76, 77 };
           private       byte[]  buffer           = new byte[ limits[ limits.length - 1 ] ];

    //----------------------------------------------------------------------------------------------
    /** Ft817calibration constructor.
     *
     *  @param  radio                           the radio's buffer.
     */
    Ft817calibration( byte[] radio ){
        int address = addresses[ 0 ];
        for ( int index = 0;  index < buffer.length;  buffer[ index++ ] = radio[ address++ ] );
    } // Ft817calibration constructor

    //----------------------------------------------------------------------------------------------
    /** getFields returns an array of the fields.  Overrides Ft___Settings.
     *
     *  @return FtField[]                       - the array of Fields.
     */
    FtField[] getFields(){
        return Field.values();
    } // Ft817calibration.getFields

    //----------------------------------------------------------------------------------------------
    /** getField returns a Field by name.  Overrides Ft___Settings.
     *
     *  @param  name                            the field name.
     *  @return FtField                         - the Field.
     */
    FtField getField( String name ){
        return Field.valueOf( name );
    } // Ft817calibration.getField

    //----------------------------------------------------------------------------------------------
    /** set updates the radio buffer from the internal buffer.  Overrides Ft___Calibration.
     *
     *  @param  radio                           the radio's buffer.
     */
    void set( byte[] radio ){
        int address = addresses[ 0 ];
        for ( int index = 0;  index < buffer.length;  radio[ address++ ] = buffer[ index++ ] );
    } // Ft817calibration.set
    
    //----------------------------------------------------------------------------------------------
    /** getBuffer returns the data buffer.  Overrides Ft___Settings.
     *
     *  @return byte[]                          - the buffer.
     */
    public byte[] getBuffer(){
        return buffer;
    } // Ft817channel.getBuffer

} // class Ft817calibration
