package net.ae7q.memory;

//==================================================================================================
/** Ft817calibration implements the options data for an FT-817.
 *
 *  @version    2006.04.09
 *  @author     Copyright © 2006 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
class Ft817options extends Ft___Options {

    //==============================================================================================
    /** Subbands describes the radio's subbands.
     */
    public enum Subbands  implements FtEnumPrefix {
        _160m (   1800,   2000 ),
        _80m  (   3500,   4000 ),
        _40m  (   7000,   7300 ),
        _30m  (  10100,  10150 ),
        _20m  (  14000,  14350 ),
        _17m  (  18068,  18168 ),
        _15m  (  21000,  21450 ),
        _12m  (  24890,  24990 ),
        _10m  (  28000,  29700 ),
        _6m   (  33000,  56000 ),
        _FM   (  76000, 108000 ),
        _AIR  ( 118000, 137000 ),
        _2m   ( 137000, 164000 ),
        _70cm ( 420000, 470000 ),
        _WIDE (    100,  33000 ),
        ;
        int lower, upper;

        //------------------------------------------------------------------------------------------
        /** Subbands constructor.
         *
         *  @param  lower                       the lower frequency limit in kHz.
         *  @param  upper                       the upper frequency limit in kHz.
         */
        Subbands( int lower, int upper ){
            this.lower = lower;
            this.upper = upper;
        } // Ft817options.Subbands constructor
    } // Ft817options.Subbands enum

    //==============================================================================================
    /** Field describes all the radio's options.
     */
    public enum Field implements FtField {
        _144MHz_Ars       ( 10, 0x40, FtSettings.Booleans.values() ),                   // Menu:01
        _430MHz_Ars       ( 10, 0x20, FtSettings.Booleans.values() ),                   // Menu:01
        _9600_Mic_Gain_Pct( 23, 0x7F,   0,   1,    0, 100 ),                            // Menu:03
        _AM_FM_Dial       ( 14, 0x80, new String[]{ "ENABLE", "DISABLE" } ),            // Menu:04
        _AM_Mic_Gain_Pct  ( 19, 0x7F,   0,   1,    0, 100 ),                            // Menu:05
//      _AM_Step          ----- In channel memory -----                                 // Menu:06
        _Antenna_HF       ( 37, 0x01, antennas ),                                       // Menu:07
        _Antenna_6M       ( 37, 0x02, antennas ),                                       // Menu:07
        _Antenna_Air      ( 37, 0x04, antennas ),                                       // Menu:07
        _Antenna_FM       ( 37, 0x08, antennas ),                                       // Menu:07
        _Antenna_VHF      ( 37, 0x10, antennas ),                                       // Menu:07
        _Antenna_UHF      ( 37, 0x20, antennas ),                                       // Menu:07
        _APO_Time_Hrs     ( 16, 0x07,   0,   1,    1,   6, new String[]{ "OFF" } ),     // Menu:08
        _ARTS_Beep        (  8, 0xC0, new String[]{ "OFF", "RANGE", "ALL" } ),          // Menu:09
        _Backlight        (  6, 0xC0, new String[]{ "OFF", "ON", "AUTO" } ),            // Menu:10
        _Battery_Chg_Hrs  ( 13, 0xC0, new String[]{ "6h", "8h", "10h" } ),              // Menu:11
        _Beep_Freq        (  7, 0x80, new String[]{ "440Hz", "880Hz" } ),               // Menu:12
        _Beep_Volume_Pct  (  7, 0x7F,   0,   1,    0, 100 ),                            // Menu:13
        _Cat_Rate         ( 15, 0xC0, new String[]{ "4800", "9600", "38400" } ),        // Menu:14
        _Color            (  6, 0x30, new String[]{ "BLUE", "AMBER", "VIOLET" } ),      // Menu:15
        _Contrast         (  6, 0x0F,  -1,   1,    2,  13 ),                            // Menu:16
        _CW_Delay_Ms      ( 11, 0xFF,   0,  10,    1, 250 ),                            // Menu:17
        _CW_ID            (  8, 0x10, FtSettings.Booleans.values() ),                   // Menu:18
        _CW_Paddle        (  3, 0x04, new String[]{ "NORMAL", "REVERSE" } ),            // Menu:19
        _CW_PitchHz       (  9, 0x0F,  50, 300,    0,  14 ),                            // Menu:20
        _CW_Speed         ( 13, 0x3F,   4,   1,    0,  56 ),                            // Menu:21
        _CW_Speed_Units   (  3, 0x08, new String[]{ "WPM", "CPM" } ),                   // Menu:21
        _CW_Weight        ( 10, 0x1F, new String[]{ "2.5", "2.6", "2.7", "2.8", "2.9",  // Menu:22
                                                    "3.0", "3.1", "3.2", "3.3", "3.4",
                                                    "3.5", "3.6", "3.7", "3.8", "3.9",
                                                    "4.0", "4.1", "4.2", "4.3", "4.4", "4.5" } ),
//      _DCS_Code         ----- In channel memory -----                                 // Menu:23
        _Dig_Disp_Hz      ( 26, 0xFFFF, 0,  10, -300, 300 ),                            // Menu:24
        _Dig_Mic_Gain_Pct ( 21, 0x7F,   0,   1,    0, 100 ),                            // Menu:25
        _Dig_Mode         ( 16, 0xE0, new String[]{ "RTTY", "PSK31-L", "PSK31-U",       // Menu:26
                                                    "USER-L", "USER-U" } ),
        _Dig_Shift_Hz     ( 24, 0xFFFF, 0,  10, -300, 300 ),                            // Menu:27
        _Emergency_Alaska ( 15, 0x20, FtSettings.Booleans.values() ),                   // Menu:28
        _FM_Mic_Gain_Pct  ( 20, 0x7F,   0,   1,    0, 100 ),                            // Menu:29
//      _FM_Step          ----- In channel memory -----                                 // Menu:30
//      _ID               ----- Unknown -----                                           // Menu:31
        _Lock_Mode        (  9, 0x30, new String[]{ "DIAL", "FREQ", "PANEL" } ),        // Menu:32
        _Main_Step        (  8, 0x20, new String[]{ "FINE", "COARSE" } ),               // Menu:33
        _Mem_Grouping     ( 16, 0x10, FtSettings.Booleans.values() ),                   // Menu:34
//      _Memory_Tag       ----- In channel memory -----                                 // Menu:35
        _Mic_Key          ( 19, 0x80, FtSettings.Booleans.values() ),                   // Menu:36
        _Mic_Scan         ( 20, 0x80, FtSettings.Booleans.values() ),                   // Menu:37
        _Op_Filter        (  9, 0xC0, new String[]{ "OFF", "SSB", "CW" } ),             // Menu:38
        _Pkt_Mic_Gain_Pct ( 22, 0x7F,   0,   1,    0, 100 ),                            // Menu:39
        _Pkt_Rate         (  8, 0x04, new String[]{ "1200bps", "9600bps" } ),           // Menu:40
        _Resume_Secs      (  8, 0x03, new String[]{ "OFF", "3", "5", "10" } ),          // Menu:41
//      _Split_Tx_Freq_Hz ----- In channel memory -----                                 // Menu:42
        _Scope            (  8, 0x08, new String[]{ "CONT", "CHK" } ),                  // Menu:43
        _Side_Tone_Pct    ( 12, 0x7F,   0,   1,    0, 100 ),                            // Menu:44
        _Squelch_Knob     ( 10, 0x80, new String[]{ "RF-GAIN", "SQL" } ),               // Menu:45
        _SSB_Mic_Gain_Pct ( 18, 0x7F,   0,   1,    0, 100 ),                            // Menu:46
//      _SSB_Step         ----- In channel memory -----                                 // Menu:47
//      _Tone_CTCSS       ----- In channel memory -----                                 // Menu:48
        _Tot_Time_Mins    ( 17, 0x1F,   0,   1,    1,  20, new String[]{ "OFF" } ),     // Menu:49
        _VOX_Delay_Ms     ( 15, 0x1F,   0, 100,    1,  25 ),                            // Menu:50
        _VOX_Gain_Pct     ( 14, 0x7F,   0,   1,    1, 100 ),                            // Menu:51
        _Extended_Menu    (  6, 0x80, FtSettings.Booleans.values() ),                   // Menu:52
        _DCS_Inv          ( 17, 0xC0, new String[]{ "TnRn","TnRiv","TivRn","TivRiv" } ),// Menu:53
        _Rx_Carrier_LSB_Hz( 28, 0xFF,   0,  10,  -30,  30 ),                            // Menu:54
        _Rx_Carrier_USB_Hz( 29, 0xFF,   0,  10,  -30,  30 ),                            // Menu:55
        _Tx_Carrier_LSB_Hz( 30, 0xFF,   0,  10,  -30,  30 ),                            // Menu:56
        _Tx_Carrier_USB_Hz( 31, 0xFF,   0,  10,  -30,  30 ),                            // Menu:57
    //  ----- Front panel settings ----- 
        _Current_Option_Menu_Index  ( 32, 0x7F, menuOptions ),                          // Knob
        _Home             (  0, 0x10, FtSettings.Booleans.values() ),                   // HOME
        _M_Tune           (  0, 0x20, FtSettings.Booleans.values() ),                   // Dial
        _MemVFO           (  0, 0xC0, new String[]{ null, "MEMORY", "VFO" } ),          // V/M
        _VFO_A_Subband    (  4, 0x0F, Subbands.values() ),                              // Band
        _VFO_B_Subband    (  4, 0xF0, Subbands.values() ),                              // Band
        _IF_Shift         (  2, 0x10, FtSettings.Booleans.values() ),                   // IF shift
        _Unlocked         (  2, 0x40, FtSettings.Booleans.values() ),                   // Lock
        _Slow_Tune_Rx     (  2, 0x80, FtSettings.Booleans.values() ),                   // Fast
        _Current_Memory_Channel_Index(  40, 0xFF, 1, 1,    0, 215 ),                    // Knob
    //  ----- MF settings ----- 
        _Current_Function_Row_Index  ( 33, 0x0F ),                                      // Knob
        _VFO_B            (  0, 0x01, FtSettings.Booleans.values() ),                   // F1:A
//      _VFO_Copy         ----- Momentary -----                                         // F1:B
        _Split_VFO        ( 37, 0x80, FtSettings.Booleans.values() ),                   // F1:C
//      _Visible          ----- In channel memory -----                                 // F2:AB
//      _Skip             ----- In channel memory -----                                 // F2:B
//      _Display_Tag      ----- In channel memory -----                                 // F2:C
//      _Memory_Store     ----- Momentary -----                                         // F3:A
        _QMB              (  0, 0x04, FtSettings.Booleans.values() ),                   // F3:B
        _MT_QMB           (  0, 0x02, FtSettings.Booleans.values() ),                   // F3:B
        _PMS_Tune_Limits  (  0, 0x08, FtSettings.Booleans.values() ),                   // F3:C
//      _Split_Tx         ----- In channel memory -----                                 // F4:A
//      _Reverse          ----- Unavailable -----                                       // F4:B
//      _Tone             ----- In channel memory -----                                 // F4:C
        _Memory_Scan      ( 36, 0x60, new String[]{"OFF",null,"SCAN UP","SCAN DOWN"} ), // F5:A
        _Dual_Scan        ( 36, 0x18, new String[]{"OFF",null,"PRIORITY","WATCH"} ),    // F5:BC
//      _Scope            ----- Unavailable -----                                       // F6:A
//      _Tone_Search      ----- Momentary -----                                         // F6:B
        _ARTS             ( 36, 0x80, FtSettings.Booleans.values() ),                   // F6:C
//      _RF_Gain          ----- In channel memory -----                                 // F7:AB
//      _Narrow_Band_FM   ----- In channel memory -----                                 // F7:C
        _Noise_Blanker    (  2, 0x20, FtSettings.Booleans.values() ),                   // F8:A
        _AGC_Mode         (  2, 0x03, new String[]{ "AUTO", "FAST", "SLOW", "OFF" } ),  // F8:C
        _RF_Power         ( 36, 0x03, new String[]{ "HIGH", "LOW3", "LOW2", "LOW1" } ), // F9:A
        _Meter_Mode       (  3, 0x03, new String[]{ "PWR", "ALC", "SWR", "MOD" } ),     // F9:B
        _VOX              (  3, 0x80, FtSettings.Booleans.values() ),                   // F10:A
        _CW_Break         (  3, 0x20, FtSettings.Booleans.values() ),                   // F10:B
        _CW_Keyer         (  3, 0x10, FtSettings.Booleans.values() ),                   // F10:C
//      _Charge           ----- Momentary -----                                         // F11:A
        _Voltage          (  3, 0x40, FtSettings.Booleans.values() ),                   // F11:B
        _Display_Large    (  2, 0x04, FtSettings.Booleans.values() ),                   // F11:C
//      _Tone_Search      ----- Momentary -----                                         // F12:C
//      _DCS_Search       ----- Momentary -----                                         // F12:C
    //  ----- Automatic settings ----- 
//      __HomeBand        (  1, 0x03, HomeBands.values() ),                             //(auto)
//      __IfShiftPriority (  2, 0x08 ),                                                 // IF shift
        ;
        private final boolean  special;
        private final Object[] keys;
        private final long     mask, base, min, max;
        private final int      offset, shift, width, scale, length;

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         */
        private Field( int offset, long mask ){
            this( offset, mask, 0, 1, 0, -1, null, false );
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  keys                        the field's keys.
         */
        private Field( int offset, long mask, Object[] keys ){
            this( offset, mask, 0, 1, 0, keys.length - 1, keys, false );
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         */
        private Field( int offset, long mask, long base, int scale ){
            this( offset, mask, base, scale, 0, -1, null, false );
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  keys                        the field's keys.
         */
        private Field( int offset, long mask, long base, int scale, Object[] keys ){
            this( offset, mask, base, scale, 0, keys.length - 1, keys, false );
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  min                         the field's minimum value.
         *  @param  max                         the field's maximum value.
         */
        private Field( int offset, long mask, long base, int scale, long min, long max ){
            this( offset, mask, base, scale, min, max, null, false );
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  min                         the field's minimum value.
         *  @param  max                         the field's maximum value.
         *  @param  keys                        the field's keys.
         */
        private Field( int offset, long mask, long base, int scale, long min, long max,
                       Object[] keys ){
            this( offset, mask, base, scale, min, max, keys, false );
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  min                         the field's minimum value.
         *  @param  max                         the field's maximum value.
         *  @param  keys                        the field's keys.
         *  @param  special                     the field's special flag.
         */
        private Field( int offset, long mask, long base, int scale, long min, long max,
                       Object[] keys, boolean special ){
            int shift, width;
            this.offset  = offset;
            this.mask    = mask;
            for ( shift = 0;  mask != 0  && (mask & 1) == 0;  shift++, mask >>>= 1 );
            for ( width = 0;  mask != 0;                      width++, mask >>>= 1 );
            this.shift   = shift;
            this.width   = width;
            this.length  = this.mask != 0 ? (shift + width - 1) / 8 + 1 : scale;
            this.base    = base;
            this.scale   = scale;
            this.min     = min;
            this.max     = max < min ? this.mask >>> shift : max;
            this.special = special;
            this.keys    = keys;
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** get retrieves the field from a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @return Object                      - the field.
         */
        public Object get( byte[] buffer ){
            if ( mask == 0 ){
                char[] temp = new char[ length ];
                for ( int index = 0;  index < length;  index++ ){
                    temp[ index ] = keys[ 0 ].toString().charAt( buffer[ offset + index ] );
                }
                return new String( temp );
            } else {
                long value = 0;
                for ( int index = 0;  index < length;  index++  ){
                    value = value << 8 | buffer[ offset + index ] & 0xFF;
                }
                value = (mask & value) >>> shift;
                if ( special ){
                    value = value >> 1 | value << 8 & 0x100;
                }
                return FtRemote.getValue( value, base, scale, min, width, keys );
            }
        } // Ft817options.Field.get

        //------------------------------------------------------------------------------------------
        /** put places a field into a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @param  object                      the field.
         */
        public void put( byte[] buffer, Object object ){
            long value;
            if ( mask == 0 ){
                String string = (String) object;
                String morse  = keys[ 0 ].toString();
                for ( int index = 0;  index < length;  index++ ){
                    value = index < string.length() ? morse.indexOf( string.charAt( index ) ) : -1;
                    buffer[ offset + index ] = (byte) (value < 0 ? morse.indexOf( ' ' ) : value);
                }
            } else {
                long mask = this.mask;
                value = FtRemote.putValue( base, scale, min, max, keys, object );
                if ( special ){
                    buffer[ offset ] = (byte) value;
                    buffer[ offset + 1 ] ^= mask & (buffer[ offset + 1 ] ^ value >> 1);
                } else {
                    value <<= shift;
                    for ( int index = length;  index-- > 0;  value >>>= 8, mask >>>= 8 ){
                        buffer[ offset + index ] ^= mask & (buffer[ offset + index ] ^ value);
                    }
                }
            }
        } // Ft817options.Field.put

        //------------------------------------------------------------------------------------------
        /** type returns the Type of the field.
         *
         *  @return Type                        - the Type of field.
         */
        public Type type(){
            return mask == 0 ? Type.MORSE : Type.OTHER;
        } // Ft817options.Field.type

    } // Ft817options.Field enum

    static private final long     serialVersionUID = 2006040900L;
    static private final int[]    addresses        = { 0x0055, 0x044F },
                                  limits           = {     40,     41 };
    static private final String[] functions
        = { "F1:A/B",   "F1:A=B",  "F1:SPL",  "F2:MW",   "F2:MC",   "F2:TAG",
            "F3:STO",   "F3:RCL",  "F3:PMS",  "F4:RPT",  "F4:REV",  "F4:TON",
            "F5:SCN",   "F5:PRI",  "F5:DW",   "F6:SSM",  "F6:SCH",  "F6:ART",
            "F7:IPO",   "F7:ATT",  "F7:NAR",  "F8:NB",   "F8:AGC",  "F8:",
            "F9:PWR",   "F9:MTR",  "F9:",    "F10:VOX", "F10:BK",  "F10:KYR",
            "F11:CHG", "F11:VLT", "F11:DSP", "F12:TCH", "F12:DCH", "F12:" };
    static private final String[] menuOptions
        = { "01:144MHz ARS", "02:430MHz ARS", "03:9600 MIC",  "04:AM&FM DIAL", "05:AM MIC GAIN",
            "06:AM STEP",    "07:ANTENNA",    "08:APO TIME",  "09:ARTS BEEP",  "10:BACKLIGHT",
            "11:BATT-CHG",   "12:BEEP FREQ",  "13:BEEP VOL",  "14:CAT RATE",   "15:COLOR",
            "16:CONTRAST",   "17:CW DELAY",   "18:CW ID",     "19:CW PADDLE",  "20:CW PITCH",
            "21:CW:SPEED",   "22:CW WEIGHT",  "23:DCS CODE",  "24:DIG DISP",   "25:DIG MIC",
            "26:DIG MODE",   "27:DIG SHIFT",  "28:EMERGENCY", "29:FM MIC",     "30:FM STEP",
            "31:ID",         "32:LOCK MODE",  "33:MAIN STEP", "34:MEM GROUP",  "35:MEM TAG",
            "36:MIC KEY",    "37:MIC SCAN",   "38:OP FILTER", "39:PKT MIC",    "40:PKT RATE",
            "41:RESUME",     "42:RPT SHIFT",  "43:SCOPE",     "44:SIDE TONE",  "45:SQL/RF-G",
            "46:SSB MIC",    "47:SSB STEP",   "48:TONE FREQ", "49:TOT TIME",   "50:VOX DELAY",
            "51:VOX GAIN",   "52:EXTEND",     "53:DCS INV",   "54:R CAR LSB",  "55:R CAR USB",
            "56:T CAR LSB",  "57:T CAR USB" };
    static private final String[] antennas = { "FRONT", "REAR" };
           private       byte[]   buffer   = new byte[ limits[ limits.length - 1 ] ];

    //----------------------------------------------------------------------------------------------
    /** Ft817options constructor.
     *
     *  @param  radio                   the buffer containing the channel data.
     */
    Ft817options( byte[] radio ){
        for ( int offset = 0, index = 0;  index < addresses.length;  index++ ){
            for ( int address = addresses[ index ];  offset < limits[ index ];
                                                     buffer[ offset++ ] = radio[ address++ ] );
        }
    } // Ft817options constructor

    //----------------------------------------------------------------------------------------------
    /** getFields returns an array of the fields.  Overrides Ft___Settings.
     *
     *  @return FtField[]                       - the array of Fields.
     */
    FtField[] getFields(){
        return Field.values();
    } // Ft817options.getFields

    //----------------------------------------------------------------------------------------------
    /** getField returns a Field by name.  Overrides Ft___Settings.
     *
     *  @param  name                            the field name.
     *  @return FtField                         - the Field.
     */
    FtField getField( String name ){
        return Field.valueOf( name );
    } // Ft817options.getField

    //----------------------------------------------------------------------------------------------
    /** set updates the radio buffer from the internal buffer.  Overrides Ft___Options.
     *
     *  @param  radio                           the radio's buffer.
     */
    void set( byte[] radio ){
        for ( int offset = 0, index = 0;  index < addresses.length;  index++ ){
            for ( int address = addresses[ index ];  offset < limits[ index ];
                                                     radio[ address++ ] = buffer[ offset++ ] );
        }
    } // Ft817options.set
    
    //----------------------------------------------------------------------------------------------
    /** getBuffer returns the data buffer.  Overrides Ft___Settings.
     *
     *  @return byte[]                          - the buffer.
     */
    public byte[] getBuffer(){
        return buffer;
    } // Ft817channel.getBuffer

} // class Ft817options
