/** Ft897D.h describes the memory layout in a Yaesu FT-897D.
 *
 *  Copyright  2005 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307	USA
 */

// Bytes are in BIG-endian order; bit fields are in LITTLE-endian order.

#include <limits.h>

#ifdef  _MSC_VER
    typedef unsigned __int8  uint8_t;
    typedef          __int8  int8_t;
    typedef unsigned __int16 uint16_t;
    typedef          __int16 int16_t;
    typedef unsigned __int32 uint32_t;
#else
#   include <stdint.h>
#endif

#define	BIT_ARRAY( bits ) ((bits-1)/CHAR_BIT + 1)

typedef struct {
    uint8_t		hf1_rxg;		// ( 1)		// 0x00/0x0007
    uint8_t		hf2_rxg;		// ( 2)		// 0x01
    uint8_t		hf3_rxg;		// ( 3)		// 0x02
    uint8_t		m50_rxg;		// ( 4)		// 0x03
    uint8_t		vhf_rxg;		// ( 5)		// 0x04
    uint8_t		uhf_rxg;		// ( 6)		// 0x05
    uint8_t		ssb_s9;			// ( 7)		// 0x06
    uint8_t		ssb_fs;			// ( 8)		// 0x07
    uint8_t		fm_s1;			// ( 9)		// 0x08
    uint8_t		fm_fs;			// (10)		// 0x09
    uint8_t		disc_l;			// (11)		// 0x0A
    uint8_t		disc_h;			// (12)		// 0x0B
    uint8_t		fm_th1;			// (13)		// 0x0C
    uint8_t		fm_th2;			// (14)		// 0x0D
    uint8_t		fm_ti1;			// (15)		// 0x0E
    uint8_t		fm_ti2;			// (16)		// 0x0F
    uint8_t		vcc;			// (17)		// 0x10
    uint8_t		hf1_ic;			// (18)		// 0x11
    uint8_t		hf2_ic;			// (19)		// 0x12
    uint8_t		hf3_ic;			// (20)		// 0x13
    uint8_t		m50_ic;			// (21)		// 0x14
    uint8_t		vhf_ic;			// (22)		// 0x15
    uint8_t		uhf_ic;			// (23)		// 0x16
    uint8_t		hf1_po_max;		// (24)		// 0x17
    uint8_t		hf1_po_mid2;		// (25)		// 0x18
    uint8_t		hf1_po_mid1;		// (26)		// 0x19
    uint8_t		hf1_po_min;		// (27)		// 0x1A
    uint8_t		hf2_po_max;		// (28)		// 0x1B
    uint8_t		hf2_po_mid2;		// (29)		// 0x1C
    uint8_t		hf2_po_mid1;		// (30)		// 0x1D
    uint8_t		hf2_po_min;		// (31)		// 0x1E
    uint8_t		hf3_po_max;		// (32)		// 0x1F
    uint8_t		hf3_po_mid2;		// (33)		// 0x20
    uint8_t		hf3_po_mid1;		// (34)		// 0x21
    uint8_t		hf3_po_min;		// (35)		// 0x22
    uint8_t		m50_po_max;		// (36)		// 0x23
    uint8_t		m50_po_mid2;		// (37)		// 0x24
    uint8_t		m50_po_mid1;		// (38)		// 0x25
    uint8_t		m50_po_min;		// (39)		// 0x26
    uint8_t		vhf_po_max;		// (40)		// 0x27
    uint8_t		vhf_po_mid;		// (41)		// 0x28
    uint8_t		vhf_po_min;		// (42)		// 0x29
    uint8_t		uhf_po_max;		// (43)		// 0x2A
    uint8_t		uhf_po_min;		// (44)		// 0x2B
    uint8_t		hf1_txg;		// (45)		// 0x2C
    uint8_t		hf2_txg;		// (46)		// 0x2D
    uint8_t		hf3_txg;		// (47)		// 0x2E
    uint8_t		m50_txg;		// (48)		// 0x2F
    uint8_t		vhf_txg;		// (49)		// 0x30
    uint8_t		uhf_txg;		// (50)		// 0x31
    uint8_t		alc1_m;			// (51)		// 0x32
    uint8_t		alc_m;			// (52)		// 0x33
    uint8_t		hf1_rev_alc;		// (53)		// 0x34
    uint8_t		hf2_rev_alc;		// (54)		// 0x35
    uint8_t		hf3_rev_alc;		// (55)		// 0x36
    uint8_t		m50_rev_alc;		// (56)		// 0x37
    uint8_t		vhf_rev_alc;		// (57)		// 0x38
    uint8_t		uhf_rev_alc;		// (58)		// 0x39
    uint8_t		cw_car_level;		// (59)		// 0x3A
    uint8_t		am_car_level;		// (60)		// 0x3B
    uint8_t		dev_w;			// (61)		// 0x3C
    uint8_t		dev_n;			// (62)		// 0x3D
    uint8_t		mod_mtr;		// (63)		// 0x3E
    uint8_t		dtmf_dev;		// (64)		// 0x3F
    uint8_t		ctcss_dev;		// (65)		// 0x40
    uint8_t		dcs_dev;		// (66)		// 0x41
    int8_t		lsb_car_point;		// (67)		// 0x42
    int8_t		usb_car_point;		// (68)		// 0x43
    uint8_t		vswr2_at_10w;		// (69)		// 0x44
    uint8_t		vswr3_at_10w;		// (70)		// 0x45
    uint8_t		atas_test;		// (71)		// 0x46
    uint8_t		amtr_test;		// (72)		// 0x47
    uint8_t		htemp_threshold;	// (73)		// 0x48
    uint8_t		ftemp_threshold;	// (74)		// 0x49
    uint8_t		tbd1[ 23 ];				// 0x4A/0x0051
} Ft897calibration;

typedef enum {
    Mode_LSB,			// 0
    Mode_USB,			// 1
    Mode_CW,			// 2
    Mode_CWR,			// 3
    Mode_AM,			// 4
    Mode_FM,			// 5
    Mode_DIG,			// 6
    Mode_PKT			// 7
} FtMode;

typedef enum {
    Band_HF,			// 0
    Band_6M,			// 1
    Band_FM,			// 2
    Band_AIR,			// 3
    Band_VHF,			// 4
    Band_UHF			// 5
} FtBand;

typedef struct {
    FtMode	mode		:3;	// Mode			// 0x00
      uint8_t			:2;	// = 0x00
      uint8_t	unknown1	:1;	// (lock?)
      uint8_t	defaultTag	:1;	// Menu:56
      uint8_t	displayTag	:1;	// MFb:C
    FtBand	band		:3;	// Band			// 0x01
      uint8_t	narrowBandFM	:1;	// MFm:C
      uint8_t			:1;	// = 0x00
      uint8_t	unknown2	:1;	// ???
      enum {
	SplitTX_SIMPLEX,		// 0
	SplitTX_MINUS,			// 1
	SplitTX_PLUS,			// 2
	SplitTX_SPL			// 3
      }		splitTX		:2;	// MFa:C
    uint8_t			:4;	// = 0x00		// 0x02
      enum {
	RfGain_PREAMP,			// 0
	RfGain_ATT,			// 1
	RfGain_IPO			// 2
      }		rfGain		:2;	// MFm:AB
      uint8_t	clarifier	:1;	// Clarifier
      uint8_t	skip		:1;	// MFb:B
    enum {
	FmStep_5kHz,			// 0
	FmStep_6_25kHz,			// 1
	FmStep_10kHz,			// 2
	FmStep_12_5kHz,			// 3
	FmStep_15kHz, 			// 4
	FmStep_20kHz, 			// 5
	FmStep_25kHz, 			// 6
	FmStep_50kHz			// 7
      }		fmStep		:3;	// Menu:52		// 0x03
      enum {
	AmStep_2_5kHz,			// 0
	AmStep_5kHz, 			// 1
	AmStep_9kHz,			// 2
	AmStep_10kHz,			// 3
	AmStep_12_5kHz, 		// 4
	AmStep_25kHz			// 5
      }		amStep		:3;	// Menu:06
      enum {
	SsbStep_1kHz,			// 0
	SsbStep_2_5kHz,			// 1
	SsbStep_5kHz			// 2
      }		ssbStep		:2;	// Menu:82
    enum {
	Code_NONE,			// 0
	Code_CTCSS,			// 1
	Code_DCS			// 2
      }		decode		:2,	// MFe:B		// 0x04
		encode		:2;	// MFe:A
      uint8_t	toneSplit	:1;	// Menu:79
      uint8_t			:3;	// = 0x00
    FtBand	splitTxBand	:3;	// Band			// 0x05
    FtMode	splitTxMode	:3;	// Mode
      uint8_t			:2;	// = 0x00
    enum {
	CTCSS_67_0,			// 0
	CTCSS_69_3,			// 1
	CTCSS_71_9,			// 2
	CTCSS_74_4,			// 3
	CTCSS_77_0,			// 4
	CTCSS_79_7,			// 5
	CTCSS_82_5,			// 6
	CTCSS_85_4,			// 7
	CTCSS_88_5,			// 8
	CTCSS_91_5,			// 9
	CTCSS_94_8,			// 10
	CTCSS_97_4,			// 11
	CTCSS_100_0,			// 12
	CTCSS_103_5,			// 13
	CTCSS_107_2,			// 14
	CTCSS_110_9,			// 15
	CTCSS_114_8,			// 16
	CTCSS_118_8,			// 17
	CTCSS_123_0,			// 18
	CTCSS_127_3,			// 19
	CTCSS_131_8,			// 20
	CTCSS_136_5,			// 21
	CTCSS_141_3,			// 22
	CTCSS_146_2,			// 23
	CTCSS_151_4,			// 24
	CTCSS_156_7,			// 25
	CTCSS_159_8,			// 26
	CTCSS_162_2,			// 27
	CTCSS_165_5,			// 28
	CTCSS_167_9,			// 29
	CTCSS_171_3,			// 30
	CTCSS_173_8,			// 31
	CTCSS_177_3,			// 32
	CTCSS_179_9,			// 33
	CTCSS_183_5,			// 34
	CTCSS_186_2,			// 35
	CTCSS_189_9,			// 36
	CTCSS_192_8,			// 37
	CTCSS_196_6,			// 38
	CTCSS_199_5,			// 39
	CTCSS_203_5,			// 40
	CTCSS_206_5,			// 41
	CTCSS_210_7,			// 42
	CTCSS_218_1,			// 43
	CTCSS_225_7,			// 44
	CTCSS_229_1,			// 45
	CTCSS_233_6,			// 46
	CTCSS_241_8,			// 47
	CTCSS_250_3,			// 48
	CTCSS_254_1			// 49
      }		encodeToneCTCSS,	// Menu:83-T		// 0x06
		decodeToneCTCSS;	// Menu:83-R		// 0x07
    enum {
	DCS_023,			// 0
	DCS_025,			// 1
	DCS_026,			// 2
	DCS_031,			// 3
	DCS_032,			// 4
	DCS_036,			// 5
	DCS_043,			// 6
	DCS_047,			// 7
	DCS_051,			// 8
	DCS_053,			// 9
	DCS_054,			// 10
	DCS_065,			// 11
	DCS_071,			// 12
	DCS_072,			// 13
	DCS_073,			// 14
	DCS_074,			// 15
	DCS_114,			// 16
	DCS_115,			// 17
	DCS_116,			// 18
	DCS_122,			// 19
	DCS_125,			// 20
	DCS_131,			// 21
	DCS_132,			// 22
	DCS_134,			// 23
	DCS_143,			// 24
	DCS_145,			// 25
	DCS_152,			// 26
	DCS_155,			// 27
	DCS_156,			// 28
	DCS_162,			// 29
	DCS_165,			// 30
	DCS_172,			// 31
	DCS_174,			// 32
	DCS_205,			// 33
	DCS_212,			// 34
	DCS_223,			// 35
	DCS_225,			// 36
	DCS_226,			// 37
	DCS_243,			// 38
	DCS_244,			// 39
	DCS_245,			// 40
	DCS_246,			// 41
	DCS_251,			// 42
	DCS_252,			// 43
	DCS_255,			// 44
	DCS_261,			// 45
	DCS_263,			// 46
	DCS_265,			// 47
	DCS_266,			// 48
	DCS_271,			// 49
	DCS_274,			// 50
	DCS_306,			// 51
	DCS_311,			// 52
	DCS_315,			// 53
	DCS_325,			// 54
	DCS_331,			// 55
	DCS_332,			// 56
	DCS_343,			// 57
	DCS_346,			// 58
	DCS_351,			// 59
	DCS_356,			// 60
	DCS_364,			// 61
	DCS_365,			// 62
	DCS_371,			// 63
	DCS_411,			// 64
	DCS_412,			// 65
	DCS_413,			// 66
	DCS_423,			// 67
	DCS_431,			// 68
	DCS_432,			// 69
	DCS_445,			// 70
	DCS_446,			// 71
	DCS_452,			// 72
	DCS_454,			// 73
	DCS_455,			// 74
	DCS_462,			// 75
	DCS_464,			// 76
	DCS_465,			// 77
	DCS_466,			// 78
	DCS_503,			// 79
	DCS_506,			// 80
	DCS_516,			// 81
	DCS_523,			// 82
	DCS_526,			// 83
	DCS_532,			// 84
	DCS_546,			// 85
	DCS_565,			// 86
	DCS_606,			// 87
	DCS_612,			// 88
	DCS_624,			// 89
	DCS_627,			// 90
	DCS_631,			// 91
	DCS_632,			// 92
	DCS_654,			// 93
	DCS_662,			// 94
	DCS_664,			// 95
	DCS_703,			// 96
	DCS_712,			// 97
	DCS_723,			// 98
	DCS_731,			// 99
	DCS_732,			// 100
	DCS_734,			// 101
	DCS_743,			// 102
	DCS_754				// 103
      }		encodeDCScode,		// Menu:33-T		// 0x08
		decodeDCScode;		// Menu:33-R		// 0x09
    int16_t	clarifierFreq10Hz;	// Clarifier		// 0x0A
    uint32_t	receiveFreq10Hz;	// Dial			// 0x0C
    uint32_t	splitTxFreq10Hz;	// Menu:76		// 0x10
    CharB	memoryTag[ 8 ];		// MFb:C		// 0x14
} Ft897channel;

typedef struct {
    uint8_t	watts			:7;	// Menu:75
    uint8_t	unknown			:1;	// Menu:75
} RfPower;

typedef struct {
    uint8_t	vfoB			:1;	// MFa:A	// 0x0068
      uint8_t	mtQmb			:1;	// MTQMB
      uint8_t	qmb			:1;	// QMB
      uint8_t	pmsTuneLimits		:1;	// MFf:C
      uint8_t	home			:1;	// HOME
      uint8_t	mTune			:1;	// MTUNE
      enum {
	MemVfo_UNUSED,				// 0
	MemVfo_MEMORY,				// 1
	MemVfo_VFO				// 2
      }		memVfo			:2;	// V/M
    enum {
	HomeBand_HF,				// 0
	HomeBand_6M,				// 1
	HomeBand_VHF,				// 2
	HomeBand_UHF,				// 3
	HomeBand_COUNT				// 4
      }		homeBand		:2;			// 0x0069
      uint8_t				:3;	// = 0x00
      uint8_t				:2;	// ???
      uint8_t				:1;	// = 0x01
    enum {
	AgcMode_AUTO,				// 0
	AgcMode_FAST,				// 1
	AgcMode_SLOW				// 2
      }		agcMode			:2;	// MFl:C	// 0x006A
      uint8_t	displayLarge		:1;	// MFi:C
      uint8_t	ifShiftPriority		:1;	// (derived)
      uint8_t	ifShift			:1;
      uint8_t	noiseBlanker		:1;	// MFl:A
      uint8_t	unlocked		:1;
      uint8_t	slowTuneRX		:1;
    enum {
	MeterMode_PWR,				// 0
	MeterMode_ALC,				// 1
	MeterMode_SWR,				// 2
	MeterMode_MOD				// 3
      }		meterMode		:2;	// MFi:B	// 0x006B
      uint8_t	cwKeyRev		:1;	// Menu:25
      uint8_t	cwSpeedInCpm		:1;	// Menu:30
      uint8_t	cwKeyer			:1;	// MFj:C
      uint8_t	cwBreak			:1;	// MFj:B
      uint8_t				:1;	// = 0x00
      uint8_t	vox			:1;	// MFd:C
    enum {
	SubBand_160m,				// 0
	SubBand_80m,				// 1
	SubBand_60m,				// 2
	SubBand_40m,				// 3
	SubBand_30m,				// 4
	SubBand_20m,				// 5
	SubBand_17m,				// 6
	SubBand_15m,				// 7
	SubBand_12m,				// 8
	SubBand_10m,				// 9
	SubBand_6m,				// 10
	SubBand_FM,				// 11
	SubBand_AIR,				// 12
	SubBand_2m,				// 13
	SubBand_70cm,				// 14
	SubBand_WIDE,				// 15
	SubBand_COUNT				// 16
      }	   	vfoAsubBand		:4,	// Band		// 0x006C
		vfoBsubBand		:4;	// Band
    uint8_t				:8;	// = 0x66	// 0x006D
    uint8_t	dispContrastPlus2	:4;	// Menu:42	// 0x006E
      uint8_t				:2;	// = 0x00
      enum {
	DispMode_OFF,				// 0
	DispMode_AUTO1,				// 1
	DispMode_AUTO2,				// 2
	DispMode_ON				// 3
      }		dispMode		:2;	// Menu:44
    enum {
	BeepTone_440Hz,				// 0
	BeepTone_880Hz,				// 1
	BeepTone_1760Hz				// 2
      }		beepTone		:2;	// Menu:13	// 0x006F
      enum {
	ClarifierKnob_MAIN,			// 0
	ClarifierKnob_VFO_MEM,			// 1
	ClarifierKnob_CLAR			// 2
      }		clarifierKnob		:2;	// Menu:21
      uint8_t				:4;	// = 0x00
    uint8_t				:2;	// = 0x00	// 0x0070
      enum {
	PktRate_1200,				// 0
	PktRate_9600				// 1
      }		pktRate			:1;	// Menu:73
      uint8_t	scopeOneSweep		:1;	// MFh:B
      uint8_t	artsId			:1;	// Menu:09
      enum {
	DialStep_FINE,				// 0
	DialStep_COARSE				// 1
      }		dialStep		:1;	// Menu:35
      enum {
	ArtsBeep_OFF,				// 0
	ArtsBeep_RANGE,				// 1
	ArtsBeep_ALL				// 2
      }		artsBeep		:2;	// Menu:08
    enum {
	CwPitch_400Hz,				// 0
	CwPitch_500Hz,				// 1
	CwPitch_600Hz,				// 2
	CwPitch_700Hz,				// 3
	CwPitch_800Hz				// 4
      }		cwPitch100HzBase4	:3;	// Menu:27	// 0x0071
      uint8_t				:1;	// = 0x00
      enum {
	LockMode_DIAL,				// 0
	LockMode_FREQ,				// 1
	LockMode_PANEL,				// 2
	LockMode_ALL				// 3
      }		lockMode		:2;	// Menu:54
      uint8_t				:2;	// = 0x00
    enum {
	CwWeight_2_5,				// 0
	CwWeight_2_6,				// 1
	CwWeight_2_7,				// 2
	CwWeight_2_8,				// 3
	CwWeight_2_9,				// 4
	CwWeight_3_0,				// 5
	CwWeight_3_1,				// 6
	CwWeight_3_2,				// 7
	CwWeight_3_3,				// 8
	CwWeight_3_4,				// 9
	CwWeight_3_5,				// 10
	CwWeight_3_6,				// 11
	CwWeight_3_7,				// 12
	CwWeight_3_8,				// 13
	CwWeight_3_9,				// 14
	CwWeight_4_0,				// 15
	CwWeight_4_1,				// 16
	CwWeight_4_2,				// 17
	CwWeight_4_3,				// 18
	CwWeight_4_4,				// 19
	CwWeight_4_5				// 20
      }		cwWeightBase2_5		:5;	// Menu:32	// 0x0072
      uint8_t	ars430MHz		:1;	// Menu:03
      uint8_t	ars144MHz		:1;	// Menu:02
      enum {
	SquelchKnob_SQL,			// 0
	SquelchKnob_RF_GAIN			// 1
      }		squelchKnob		:1;	// Menu:80
    uint8_t	cwDelay10ms;			// Menu:24	// 0x0073
    uint8_t	cwSideTonePct		:7;	// Menu:29	// 0x0074
      uint8_t	cwDelay10msHiBit	:1;	// Menu:24
    uint8_t	cwSpeedBase4wpm		:6;	// Menu:30	// 0x0075
      uint8_t				:2;	// = 0x01
    uint8_t	voxGainPct		:7;	// Menu:88	// 0x0076
      uint8_t	amFmDialdisable		:1;	// Menu:04
    uint8_t	voxDelay100ms		:5;	// Menu:87	// 0x0077
      uint8_t	emergencyAlaska		:1;	// Menu:50
      uint8_t				:2;	// = 0x00
    uint8_t	apoTimeHrs		:3;	// Menu:07	// 0x0078
      uint8_t				:1;	// = 0x00
      uint8_t	memGrouping		:1;	// Menu:55
      enum {
	DigMode_RTTY_L,				// 0
	DigMode_RTTY_U,				// 1
	DigMode_PSK31_L,			// 2
	DigMode_PSK31_U,			// 3
	DigMode_USER_L,				// 4
	DigMode_USER_U				// 5
      }		digMode			:3;	// Menu:38
    uint8_t	txTimeMins		:5;	// Menu:84	// 0x0079
      uint8_t				:1;	// = 0x00
      enum {
	DcsInv_TnRn,				// 0
	DcsInv_TnRiv,				// 1
	DcsInv_TivRn,				// 2
	DcsInv_TivRiv				// 3
      uint8_t	dcsInv			:2;	// Menu:34
    uint8_t	ssbMicGainPct		:7;	// Menu:81	// 0x007A
      uint8_t	micScan			:1;	// Menu:58
    uint8_t	amMicGainPct		:7;	// Menu:05	// 0x007B
      uint8_t	cwMicKey		:1;	// Menu:26
    uint8_t	fmMicGainPct;			// Menu:51	// 0x007C
    uint8_t	digGainPct;			// Menu:37	// 0x007D
    uint8_t	pkt1200gainPct;			// Menu:71	// 0x007E
    uint8_t	pkt9600gainPct;			// Menu:72	// 0x007F
    int16_t	digShift10Hz;			// Menu:39	// 0x0080
    int16_t	digDisp10Hz;			// Menu:36	// 0x0082
    int8_t	carrierLsbRx10Hz;		// Menu:15	// 0x0084
    int8_t	carrierUsbRx10Hz;		// Menu:17	// 0x0085
    int8_t	carrierLsbTx10Hz;		// Menu:16	// 0x0086
    int8_t	carrierUsbTx10Hz;		// Menu:18	// 0x0087
    enum {
	Menu_01_EXT_MENU,			// 0
	Menu_02_ARS_144MHz,			// 1
	Menu_03_ARS_430MHz,			// 2
	Menu_04_AM_FM_DIAL,			// 3
	Menu_05_AM_MIC_GAIN,			// 4
	Menu_06_AM_STEP,			// 5
	Menu_07_APO_TIME,			// 6
	Menu_08_ARTS_BEEP,			// 7
	Menu_09_ARTS_ID,			// 8
	Menu_10_ARTS_IDW,			// 9
	Menu_11_BEACON_TEXT,			// 10
	Menu_12_BEACON_TIME,			// 11
	Menu_13_BEEP_TONE,			// 12
	Menu_14_BEEP_VOL,			// 13
	Menu_15_CAR_LSB_R,			// 14
	Menu_16_CAR_LSB_T,			// 15
	Menu_17_CAR_USB_R,			// 16
	Menu_18_CAR_USB_T,			// 17
	Menu_19_CAT_RATE,			// 18
	Menu_20_CAT_LIN_TUN,			// 19
	Menu_21_CLAR_DIAL,			// 20
	Menu_22_CW_AUTO,			// 21
	Menu_23_CW_BFO,				// 22
	Menu_24_CW_DELAY,			// 23
	Menu_25_CW_KEY_REV,			// 24
	Menu_26_CW_PADDLE,			// 25
	Menu_27_CW_PITCH,			// 26
	Menu_28_CW_QSK,				// 27
	Menu_29_CW_SIDETONE,			// 28
	Menu_30_CW_SPEED,			// 29
	Menu_31_CW_TRAIN,			// 30
	Menu_32_CW_WEIGH,			// 31
	Menu_33_DCS_CODE,			// 32
	Menu_34_DCS_INV,			// 33
	Menu_35_DIAL_STEP,			// 34
	Menu_36_DIG_DISP,			// 35
	Menu_37_DIG_GAIN,			// 36
	Menu_38_DIG_MODE,			// 37
	Menu_39_DIG_SHIFT,			// 38
	Menu_40_DIG_VOX,			// 39
	Menu_41_DISP_COLOR,			// 40
	Menu_42_DISP_CONTRAST,			// 41
	Menu_43_DISP_INTENSITY,			// 42
	Menu_44_DISP_MODE,			// 43
	Menu_45_DSP_BPF,			// 44
	Menu_46_DSP_HPF,			// 45
	Menu_47_DSP_LPF,			// 46
	Menu_48_DSP_MIC_EQ,			// 47
	Menu_49_DSP_NR_LEVEL,			// 48
	Menu_50_EMERGENCY,			// 49
	Menu_51_FM_MIC_GAIN,			// 50
	Menu_52_FM_STEP,			// 51
	Menu_53_HOME_VFO,			// 52
	Menu_54_LOCK_MODE,			// 53
	Menu_55_MEM_GROUP,			// 54
	Menu_56_MEM_TAG,			// 55
	Menu_57_MEM_VFO_DIAL,			// 56
	Menu_58_MIC_SCAN,			// 57
	Menu_59_MIC_SEL,			// 58
	Menu_60_MTR_ARX,			// 59
	Menu_61_MTR_ATX,			// 60
	Menu_62_MTR_PEAK_HOLD,			// 61
	Menu_63_NB_LEVEL,			// 62
	Menu_64_OP_FILTER,			// 63
	Menu_65_PG_A,				// 64
	Menu_66_PG_B,				// 65
	Menu_67_PG_C,				// 66
	Menu_68_PG_ACC,				// 67
	Menu_69_PG_P1,				// 68
	Menu_70_PG_P2,				// 69
	Menu_71_PKT_1200,			// 70
	Menu_72_PKT_9600,			// 71
	Menu_73_PKT_RATE,			// 72
	Menu_74_PROC_LEVEL,			// 73
	Menu_75_RF_POWER,			// 74
	Menu_76_RPT_SHIFT,			// 75
	Menu_77_SCAN_MODE,			// 76
	Menu_78_SCAN_RESUME,			// 77
	Menu_79_SPLIT_TONE,			// 78
	Menu_80_SQL_RF_GAIN,			// 79
	Menu_81_SSB_MIC_GAIN,			// 80
	Menu_82_SSB_STEP,			// 81
	Menu_83_TONE_FREQ,			// 82
	Menu_84_TX_TIME,			// 83
	Menu_85_TUNER_ATAS,			// 84
	Menu_86_TX_IF_FILTER,			// 85
	Menu_87_VOX_DELAY,			// 86
	Menu_88_VOX_GAIN,			// 87
	Menu_89_XVTR_A_FREQ,			// 88
	Menu_90_XVTR_B_FREQ,			// 89
	Menu_91_XVTR_SEL			// 90
      }		currentOptionMenuIndex;		// Knob		// 0x0088
    uint8_t	currentFunctionRowIndex		// Knob		// 0x0089
    uint8_t			:8;		// = 0x00	// 0x008A
    uint8_t			:8;		// = 0x00	// 0x008B
    uint8_t				:3;	// = 0x00	// 0x008C
      enum {
	DualScan_OFF,				// 0
	DualScan_UNUSED,			// 1
	DualScan_PRIORITY,			// 2
	DualScan_WATCH				// 3
      }		dualScan		:2;	// MFf:BC
      uint8_t				:1;	// = 0x00
      uint8_t	memoryScan		:1;	// MFg:A
      uint8_t	arts			:1;	// MFf:A
    uint8_t	mh59dial		:1;	// MH59dial	// 0x008D
      uint8_t	mh59afGain		:1;	// MH59dial
      uint8_t				:5;	// = 0x00
      uint8_t	splitVfo		:1;	// MFa:C
    uint8_t				:2;	// = 0x00	// 0x008E
      enum {
	CatLinTun_CAT,				// 0
	CatLinTun_LINEAR,			// 1
	CatLinTun_TUNER				// 2
      }		catLinTun		:2;	// Menu:20
      enum {
	MicSel_NOR,				// 0
	MicSel_RMT,				// 1
	MicSel_CAT				// 2
      }		micSel			:2;	// Menu:59
      uint8_t	mtrPeakHold		:1;	// Menu:62
      uint8_t				:1;	// = 0x00
    uint8_t	beepVolumePct;			// Menu:14	// 0x008F
    uint8_t	digVoxPct;			// Menu:40	// 0x0090
    uint8_t	scanResumeSecs		:4;	// Menu:78	// 0x0091
      enum {
	ScanMode_TIME,				// 0
	ScanMode_BUSY,				// 1
	ScanMode_STOP				// 2
      }		scanMode		:2;	// Menu:77
      uint8_t	homeToVFO		:1;	// Menu:53
      uint8_t	extendedMenu		:1;	// Menu:01
    enum {
	CwBFO_USB,				// 0
	CwBFO_LSB,				// 1
	CwBFO_AUTO				// 2
      }		cwBFO			:2;	// Menu:23	// 0x0092
      enum {
	CwQSK_10ms,				// 0
	CwQSK_15ms,				// 1
	CwQSK_20ms,				// 2
	CwQSK_25ms,				// 3
	CwQSK_30ms				// 4
      }		cwQSK			:3;	// Menu:28
      enum {
	CwTrain_N,				// 0
	CwTrain_A,				// 1
	CwTrain_AN				// 2
      }		cwTrain			:2;	// Menu:31
      uint8_t	cwAuto			:1;	// Menu:22
    enum {
	DispMicEq_OFF,				// 0
	DispMicEq_LPF,				// 1
	DispMicEq_HPF,				// 2
	DispMicEq_BOTH				// 3
      }		dspMicEqualize		:2;	// Menu:48	// 0x0093
      enum {
	DspBPF_60Hz,				// 0
	DspBPF_120Hz,				// 1
	DspBPF_240Hz				// 2
      }		dspCwBPF		:2;	// Menu:45
      uint8_t	dspNRbase1		:4;	// Menu:49
    enum {
	DspLPF_1000Hz,				// 0
	DspLPF_1160Hz,				// 1
	DspLPF_1320Hz,				// 2
	DspLPF_1480Hz,				// 3
	DspLPF_1650Hz,				// 4
	DspLPF_1800Hz,				// 5
	DspLPF_1970Hz,				// 6
	DspLPF_2130Hz,				// 7
	DspLPF_2290Hz,				// 8
	DspLPF_2450Hz,				// 9
	DspLPF_2610Hz,				// 10
	DspLPF_2770Hz,				// 11
	DspLPF_2940Hz,				// 12
	DspLPF_3100Hz,				// 13
	DspLPF_3260Hz,				// 14
	DspLPF_3420Hz,				// 15
	DspLPF_3580Hz,				// 16
	DspLPF_3740Hz,				// 17
	DspLPF_3900Hz,				// 18
	DspLPF_4060Hz,				// 19
	DspLPF_4230Hz,				// 20
	DspLPF_4390Hz,				// 21
	DspLPF_4550Hz,				// 22
	DspLPF_4710Hz,				// 23
	DspLPF_4870Hz,				// 24
	DspLPF_5030Hz,				// 25
	DspLPF_5190Hz,				// 26
	DspLPF_5390Hz,				// 27
	DspLPF_5520Hz,				// 28
	DspLPF_5680Hz,				// 29
	DspLPF_5840Hz,				// 30
	DspLPF_6000Hz				// 31
      }		dspLPF			:5;	// Menu:47	// 0x0094
      uint8_t				:1;	// = 0x00
      enum {
	BaudRate_4800,				// 0
	BaudRate_9600,				// 1
	BaudRate_38400				// 2
      }		baudRate		:2;	// Menu:19
    enum {
	DspHPF_100Hz,				// 0
	DspHPF_160Hz,				// 1
	DspHPF_220Hz,				// 2
	DspHPF_280Hz,				// 3
	DspHPF_340Hz,				// 4
	DspHPF_400Hz,				// 5
	DspHPF_460Hz,				// 6
	DspHPF_520Hz,				// 7
	DspHPF_580Hz,				// 8
	DspHPF_640Hz,				// 9
	DspHPF_720Hz,				// 10
	DspHPF_760Hz,				// 11
	DspHPF_820Hz,				// 12
	DspHPF_880Hz,				// 13
	DspHPF_940Hz,				// 14
	DspHPF_1000Hz				// 15
      }		dspHPF			:4;	// Menu:46	// 0x0095
      uint8_t				:1;	// = 0x01
      enum {
	MtrATxSel_PWR,				// 0
	MtrATxSel_ALC,				// 1
	MtrATxSel_MOD,				// 2
	MtrATxSel_SWR,				// 3
	MtrATxSel_VLT,				// 4
	MtrATxSel_NA,				// 5
	MtrATxSel_OFF				// 6
      }		mtrATxSel		:3;	// Menu:61
    enum {
	DispColor_ARTS,				// 0
	DispColor_BAND,				// 1
	DispColor_FIX,				// 2
	DispColor_MEMGRP,			// 3
	DispColor_MODE,				// 4
	DispColor_MTR,				// 5
	DispColor_VFO				// 6
      }		dispColor		:3;	// Menu:41	// 0x0096
      uint8_t				:1;	// = 0x00
      uint8_t	dispIntensity		:2;	// Menu:43
      uint8_t				:2;	// = 0x00
    uint8_t	colorARTS		:1;	// Menu:41	// 0x0097
      uint8_t	colorBand		:1;	// Menu:41
      uint8_t				:1;	// = 0x00
      uint8_t	colorMemGrp		:1;	// Menu:41
      uint8_t	colorMode		:1;	// Menu:41
      uint8_t	colorMtr		:1;	// Menu:41
      uint8_t	colorVFO		:1;	// Menu:41
      uint8_t				:1;	// = 0x00
    uint8_t	colorFixBase1		:5;	// Menu:41	// 0x0098
      uint8_t				:3;	// = 0x00
    uint8_t	nbLevelPct;			// Menu:63	// 0x0099
    uint8_t	procLevelPct;			// Menu:74	// 0x009A
    RfPower	rfPowerHF;			// Menu:75	// 0x009B
    enum {
	MemVfoKnobMode_CW_SIDETONE,		// 0
	MemVfoKnobMode_CW_SPEED,		// 1
	MemVfoKnobMode_MHz_MEMGRP,		// 2
	MemVfoKnobMode_MIC_GAIN,		// 3
	MemVfoKnobMode_NB_LEVEL,		// 4
	MemVfoKnobMode_RF_POWER,		// 5
	MemVfoKnobMode_STEP			// 6
      }		memVfoKnobMode		:3;	// Menu:57	// 0x009C
      enum {
	TuneMode_OFF,				// 0
	TuneMode_ATAS_HF,			// 1
	TuneMode_ATAS_HF_50,			// 2
	TuneMode_ATAS_ALL,			// 3
	TuneMode_TUNER				// 4
      }		tuneMode		:3;	// Menu:85
      uint8_t				:2;	// = 0x00
    enum {
	ProgrammableOption_MFa_A_B,		// 0
	ProgrammableOption_MFa_AeqB,		// 1
	ProgrammableOption_MFa_SPL,		// 2
	ProgrammableOption_MFb_MW,		// 3
	ProgrammableOption_MFb_SKIP_MCLR,	// 4
	ProgrammableOption_MFb_TAG,		// 5
	ProgrammableOption_MFc_STO,		// 6
	ProgrammableOption_MFc_RCL,		// 7
	ProgrammableOption_MFc_PROC,		// 8
	ProgrammableOption_MFd_RPT,		// 9
	ProgrammableOption_MFd_REV,		// 10
	ProgrammableOption_MFd_VOX,		// 11
	ProgrammableOption_MFe_TON_ENC,		// 12
	ProgrammableOption_MFe_TON_DEC,		// 13
	ProgrammableOption_MFe_TDCH,		// 14
	ProgrammableOption_MFf_ARTS,		// 15
	ProgrammableOption_MFf_SRCH,		// 16
	ProgrammableOption_MFf_PMS,		// 17
	ProgrammableOption_MFg_SCN,		// 18
	ProgrammableOption_MFg_PRI,		// 19
	ProgrammableOption_MFg_DW,		// 20
	ProgrammableOption_MFh_SCOP,		// 21
	ProgrammableOption_MFh_WID,		// 22
	ProgrammableOption_MFh_STEP,		// 23
	ProgrammableOption_MFi_MTR,		// 24
	ProgrammableOption_MFi_SWR,		// 25
	ProgrammableOption_MFi_DISP,		// 26
	ProgrammableOption_MFj_SPOT,		// 27
	ProgrammableOption_MFj_BK,		// 28
	ProgrammableOption_MFj_KYR,		// 29
	ProgrammableOption_MFk_TUNE,		// 30
	ProgrammableOption_MFk_DOWN,		// 31
	ProgrammableOption_MFk_UP,		// 32
	ProgrammableOption_MFl_NB,		// 33
	ProgrammableOption_MFl_AGC,		// 34
	ProgrammableOption_MFl_AGC_SEL,		// 35
	ProgrammableOption_MFm_IPO,		// 36
	ProgrammableOption_MFm_ATT,		// 37
	ProgrammableOption_MFm_NAR,		// 38
	ProgrammableOption_MFn_CFIL,		// 39
	ProgrammableOption_MFn_FIL1,		// 40
	ProgrammableOption_MFn_FIL2,		// 41
	ProgrammableOption_MFo_PLY1,		// 42
	ProgrammableOption_MFo_PLY2,		// 43
	ProgrammableOption_MFo_PLY3,		// 44
	ProgrammableOption_MFp_DNR,		// 45
	ProgrammableOption_MFp_DNF,		// 46
	ProgrammableOption_MFp_DBF,		// 47
	ProgrammableOption_01_EXT_MENU,		// 48
	ProgrammableOption_02_ARS_144MHz,	// 49
	ProgrammableOption_03_ARS_430MHz,	// 50
	ProgrammableOption_04_AM_FM_DIAL,	// 51
	ProgrammableOption_05_AM_MIC_GAIN,	// 52
	ProgrammableOption_06_AM_STEP,		// 53
	ProgrammableOption_07_APO_TIME,		// 54
	ProgrammableOption_08_ARTS_BEEP,	// 55
	ProgrammableOption_09_ARTS_ID,		// 56
	ProgrammableOption_10_ARTS_IDW,		// 57
	ProgrammableOption_11_BEACON_TEXT,	// 58
	ProgrammableOption_12_BEACON_TIME,	// 59
	ProgrammableOption_13_BEEP_TONE,	// 60
	ProgrammableOption_14_BEEP_VOL,		// 61
	ProgrammableOption_15_CAR_LSB_R,	// 62
	ProgrammableOption_16_CAR_LSB_T,	// 63
	ProgrammableOption_17_CAR_USB_R,	// 64
	ProgrammableOption_18_CAR_USB_T,	// 65
	ProgrammableOption_19_CAT_RATE,		// 66
	ProgrammableOption_20_CAT_LIN_TUN,	// 67
	ProgrammableOption_21_CLAR_DIAL,	// 68
	ProgrammableOption_22_CW_AUTO,		// 69
	ProgrammableOption_23_CW_BFO,		// 70
	ProgrammableOption_24_CW_DELAY,		// 71
	ProgrammableOption_25_CW_KEY_REV,	// 72
	ProgrammableOption_26_CW_PADDLE,	// 73
	ProgrammableOption_27_CW_PITCH,		// 74
	ProgrammableOption_28_CW_QSK,		// 75
	ProgrammableOption_29_CW_SIDETONE,	// 76
	ProgrammableOption_30_CW_SPEED,		// 77
	ProgrammableOption_31_CW_TRAIN,		// 78
	ProgrammableOption_32_CW_WEIGH,		// 79
	ProgrammableOption_33_DCS_CODE,		// 80
	ProgrammableOption_34_DCS_INV,		// 81
	ProgrammableOption_35_DIAL_STEP,	// 82
	ProgrammableOption_36_DIG_DISP,		// 83
	ProgrammableOption_37_DIG_GAIN,		// 84
	ProgrammableOption_38_DIG_MODE,		// 85
	ProgrammableOption_39_DIG_SHIFT,	// 86
	ProgrammableOption_40_DIG_VOX,		// 87
	ProgrammableOption_41_DISP_COLOR,	// 88
	ProgrammableOption_42_DISP_CONTRAST,	// 89
	ProgrammableOption_43_DISP_INTENSITY,	// 90
	ProgrammableOption_44_DISP_MODE,	// 91
	ProgrammableOption_45_DSP_BPF,		// 92
	ProgrammableOption_46_DSP_HPF,		// 93
	ProgrammableOption_47_DSP_LPF,		// 94
	ProgrammableOption_48_DSP_MIC_EQ,	// 95
	ProgrammableOption_49_DSP_NR_LEVEL,	// 96
	ProgrammableOption_50_EMERGENCY,	// 97
	ProgrammableOption_51_FM_MIC_GAIN,	// 98
	ProgrammableOption_52_FM_STEP,		// 99
	ProgrammableOption_53_HOME_VFO,		// 100
	ProgrammableOption_54_LOCK_MODE,	// 101
	ProgrammableOption_55_MEM_GROUP,	// 102
	ProgrammableOption_56_MEM_TAG,		// 103
	ProgrammableOption_57_MEM_VFO_DIAL,	// 104
	ProgrammableOption_58_MIC_SCAN,		// 105
	ProgrammableOption_59_MIC_SEL,		// 106
	ProgrammableOption_60_MTR_ARX,		// 107
	ProgrammableOption_61_MTR_ATX,		// 108
	ProgrammableOption_62_MTR_PEAK_HOLD,	// 109
	ProgrammableOption_63_NB_LEVEL,		// 110
	ProgrammableOption_64_OP_FILTER,	// 111
	ProgrammableOption_71_PKT_1200,		// 112
	ProgrammableOption_72_PKT_9600,		// 113
	ProgrammableOption_73_PKT_RATE,		// 114
	ProgrammableOption_74_PROC_LEVEL,	// 115
	ProgrammableOption_75_RF_POWER,		// 116
	ProgrammableOption_76_RPT_SHIFT,	// 117
	ProgrammableOption_77_SCAN_MODE,	// 118
	ProgrammableOption_78_SCAN_RESUME,	// 119
	ProgrammableOption_79_SPLIT_TONE,	// 120
	ProgrammableOption_80_SQL_RF_GAIN,	// 121
	ProgrammableOption_81_SSB_MIC_GAIN,	// 122
	ProgrammableOption_82_SSB_STEP,		// 123
	ProgrammableOption_83_TONE_FREQ,	// 124
	ProgrammableOption_84_TX_TIME,		// 125
	ProgrammableOption_85_TUNER_ATAS,	// 126
	ProgrammableOption_86_TX_IF_FILTER,	// 127
	ProgrammableOption_87_VOX_DELAY,	// 128
	ProgrammableOption_88_VOX_GAIN,		// 129
	ProgrammableOption_89_XVTR_A_FREQ,	// 130
	ProgrammableOption_90_XVTR_B_FREQ,	// 131
	ProgrammableOption_91_XVTR_SEL,		// 132
	ProgrammableOption_MONI,		// 133
	ProgrammableOption_Q_SPL,		// 134
	ProgrammableOption_TCALL,		// 135
	ProgrammableOption_ATC,			// 136
	ProgrammableOption_USER			// 137
      }		programmableOptionA,		// Menu:65	// 0x009D
		programmableOptionB,		// Menu:66	// 0x009E
		programmableOptionC,		// Menu:67	// 0x009F
		programmableOptionAcc,		// Menu:68	// 0x00A0
		programmableOptionP1,		// Menu:69	// 0x00A1
		programmableOptionP2;		// Menu:70	// 0x00A2
    uint8_t				:3;	// = 0x00	// 0x00A3
      uint8_t	xvtrSel			:2;	// Menu:91
      uint8_t				:3;	// = 0x03
    uint8_t	txIfFilter		:2;	// Menu:86	// 0x00A4
      uint8_t				:6;	// = 0x01
    enum {
	MtrARxSel_SIG,				// 0
	MtrARxSel_CTR,				// 1
	MtrARxSel_VLT,				// 2
	MtrARxSel_NA,				// 3
	MtrARxSel_FS,				// 4
	MtrARxSel_OFF				// 5
      }		mtrARxSel		:3;	// Menu:60	// 0x00A5
      uint8_t	xvtrBnegative		:1;	// Menu:90
      uint8_t	xvtrAnegative		:1;	// Menu:89
      uint8_t				:3;	// = 0x03
    uint8_t	beaconTimeSecs;			// Menu:12	// 0x00A6
    uint8_t	rxSsbFilter		:2;	// MFn:ABC	// 0x00A7
      uint8_t	rxCwFilter		:2;	// MFn:ABC
      uint8_t	rxDigFilter		:2;	// MFn:ABC
      uint8_t				:1;	// = 0x00
      uint8_t	rxNarrowFilter		:1;	// (derived)
    uint8_t	dspDNF			:1;	// MFp:B	// 0x00A8
      uint8_t	dspDNR			:1;	// MFp:A
      uint8_t	dspDBF			:2;	// MFp:C (both on)
      uint8_t				:1;	// = 0x00
      uint8_t	agc			:1;	// MFl:B
      uint8_t				:2;	// = 0x00
    uint8_t				:1;	// = 0x00	// 0x00A9
      uint8_t	speechProcessor		:1;	// MFc:C
      enum {
	ScopeChannels_21,			// 0
	ScopeChannels_31,			// 1
	ScopeChannels_127			// 2
      }		scopeChannels		:2;	// MFh:B
      uint8_t	scopePeakHold		:1;	// MFh:C
      uint8_t				:2;	// = 0x00
      uint8_t	digVox			:1;	// Menu:40
    RfPower	rfPower6M,			// Menu:75	// 0x00AA
		rfPowerVHF,			// Menu:75	// 0x00AB
		rfPowerUHF;			// Menu:75	// 0x00AC
    uint8_t				:7;	// = 0x28	// 0x00AD
      uint8_t	tuner			:1;	// MFk:A
} Ft897options;

typedef enum {
    Morse_0,					// 0
    Morse_1,					// 1
    Morse_2,					// 2
    Morse_3,					// 3
    Morse_4,					// 4
    Morse_5,					// 5
    Morse_6,					// 6
    Morse_7,					// 7
    Morse_8,					// 8
    Morse_9,					// 9
    Morse_A,					// 10
    Morse_B,					// 11
    Morse_C,					// 12
    Morse_D,					// 13
    Morse_E,					// 14
    Morse_F,					// 15
    Morse_G,					// 16
    Morse_H,					// 17
    Morse_I,					// 18
    Morse_J,					// 19
    Morse_K,					// 20
    Morse_L,					// 21
    Morse_M,					// 22
    Morse_N,					// 23
    Morse_O,					// 24
    Morse_P,					// 25
    Morse_Q,					// 26
    Morse_R,					// 27
    Morse_S,					// 28
    Morse_T,					// 29
    Morse_U,					// 30
    Morse_V,					// 31
    Morse_W,					// 32
    Morse_X,					// 33
    Morse_Y,					// 34
    Morse_Z,					// 35
    Morse_SPACE,				// 36
    Morse_SLANT,				// 37
    Morse_END,					// 38
    Morse_MORE,					// 39
    Morse_PERIOD				// 40
} Morse;

typedef struct {
    uint8_t		header[ 4 ];		// = 0xA55AA55A	// 0x0000
    uint16_t		id;			// = 0x1F06	// 0x0004
    uint8_t		filler;			// = 0xA5	// 0x0006
    Ft897calibration	calibration;				// 0x0007
    Ft897options	options;				// 0x0068
    uint8_t		menuVisible[ BIT_ARRAY( 91 ) ];		// 0x00AE
    Ft897channel	vfoA[ SubBand_COUNT ];			// 0x00BA
    Ft897channel	vfoB[ SubBand_COUNT ];			// 0x027A
    Ft897channel	home[ HomeBand_COUNT ];			// 0x043A
    Ft897channel	qmb;					// 0x04AA
    Ft897channel	mtQmb;					// 0x04C6
    Ft897channel	mTune;					// 0x04E2
    uint8_t		tbd2[ 16 ];				// 0x04FE
    uint8_t		currentMemoryChannelIndex;		// 0x050E
    uint8_t		channelVisible[ BIT_ARRAY( 211 ) ];	// 0x050F
    uint8_t		channelAssigned[ BIT_ARRAY( 210 ) ];	// 0x052A
    Ft897channel	channels[ 200 ];			// 0x0545
    struct {
	Ft897channel	low;
	Ft897channel	high;
    }			limitSets[ 5 ];				// 0x1B25
    Ft897channel	emergencyAlaska;			// 0x1C3D
    Morse		artsIdText[ 10 ];	// Menu:10	// 0x1C59
    Morse		beaconPlay1[ 40 ];	// Menu:11-1	// 0x1C63
    Morse		beaconPlay2[ 40 ];	// Menu:11-2	// 0x1C8B
    Morse		beaconPlay3[ 40 ];	// Menu:11-3	// 0x1CB3
    uint32_t		xvtrAoffset10Hz;	// Menu:89	// 0x1CDB
    uint32_t		xvtrBoffset10Hz;	// Menu:90	// 0x1CDF
    Morse		opFilterName1[ 4 ];	// Menu:64-1	// 0x1CE3
    Morse		opFilterName2[ 4 ];	// Menu:64-2	// 0x1CE7
    uint16_t		checksum;  // ?? Chng every power cycle	// 0x1CEB
    uint8_t		tbd3[ 6 ];				// 0x1CED
} Ft897map;

static char const morse[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ /!+.";
