package net.ae7q.memory;

//==================================================================================================
/** Ft897calibration implements the calibration data for an FT-897.
 *
 *  @version    2006.04.09
 *  @author     Copyright © 2006 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
class Ft897calibration extends Ft___Calibration {

    //==============================================================================================
    /** Field describes all the radio's calibration settings.
     */
    public enum Field implements FtField, FtEnumPrefix {
        _HF1_RxG,               // ( 1) // 0x00
        _HF2_RxG,               // ( 2) // 0x01
        _HF3_RxG,               // ( 3) // 0x02
        _50m_RxG,               // ( 4) // 0x03
        _VHF_RxG,               // ( 5) // 0x04
        _UHF_RxG,               // ( 6) // 0x05
        _SSB_S9,                // ( 7) // 0x06
        _SSB_FS,                // ( 8) // 0x07
        _FM_S1,                 // ( 9) // 0x08
        _FM_FS,                 // (10) // 0x09
        _Disc_L,                // (11) // 0x0A
        _Disc_H,                // (12) // 0x0B
        _Fm_Th1,                // (13) // 0x0C
        _Fm_Th2,                // (14) // 0x0D
        _Fm_Ti1,                // (15) // 0x0E
        _Fm_Ti2,                // (16) // 0x0F
        _VCC,                   // (17) // 0x10
        _HF1_Ic,                // (18) // 0x11
        _HF2_Ic,                // (19) // 0x12
        _HF3_Ic,                // (20) // 0x13
        _50m_Ic,                // (21) // 0x14
        _VHF_Ic,                // (22) // 0x15
        _UHF_Ic,                // (23) // 0x16
        _HF1_Po_Max,            // (24) // 0x17
        _HF1_Po_Mid2,           // (25) // 0x18
        _HF1_Po_Mid1,           // (26) // 0x19
        _HF1_Po_Min,            // (27) // 0x1A
        _HF2_Po_Max,            // (28) // 0x1B
        _HF2_Po_Mid2,           // (29) // 0x1C
        _HF2_Po_Mid1,           // (30) // 0x1D
        _HF2_Po_Min,            // (31) // 0x1E
        _HF3_Po_Max,            // (32) // 0x1F
        _HF3_Po_Mid2,           // (33) // 0x20
        _HF3_Po_Mid1,           // (34) // 0x21
        _HF3_Po_Min,            // (35) // 0x22
        _50m_Po_Max,            // (36) // 0x23
        _50m_Po_Mid2,           // (37) // 0x24
        _50m_Po_Mid1,           // (38) // 0x25
        _50m_Po_Min,            // (39) // 0x26
        _VHF_Po_Max,            // (40) // 0x27
        _VHF_Po_Mid,            // (41) // 0x28
        _VHF_Po_Min,            // (42) // 0x29
        _UHF_Po_Max,            // (43) // 0x2A
        _UHF_Po_Min,            // (44) // 0x2B
        _HF1_TxG,               // (45) // 0x2C
        _HF2_TxG,               // (46) // 0x2D
        _HF3_TxG,               // (47) // 0x2E
        _50m_TxG,               // (48) // 0x2F
        _VHF_TxG,               // (49) // 0x30
        _UHF_TxG,               // (50) // 0x31
        _ALC1_M,                // (51) // 0x32
        _ALC_M,                 // (52) // 0x33
        _HF1_Rev_Alc,           // (53) // 0x34
        _HF2_Rev_Alc,           // (54) // 0x35
        _HF3_Rev_Alc,           // (55) // 0x36
        _50m_Rev_Alc,           // (56) // 0x37
        _VHF_Rev_Alc,           // (57) // 0x38
        _UHF_Rev_Alc,           // (58) // 0x39
        _CW_Car_Level,          // (59) // 0x3A
        _AM_Car_Level,          // (60) // 0x3B
        _Dev_W,                 // (61) // 0x3C
        _Dev_N,                 // (62) // 0x3D
        _Mod_Mtr,               // (63) // 0x3E
        _DTMF_Dev,              // (64) // 0x3F
        _CTCSS_Dev,             // (65) // 0x40
        _DCS_Dev,               // (66) // 0x41
        _LSB_Car_Point ( true ),// (67) // 0x42
        _USB_Car_Point ( true ),// (68) // 0x43
        _Vswr2_at_10W,          // (69) // 0x44
        _Vswr3_at_10W,          // (70) // 0x45
        _ATAS_test,             // (71) // 0x46
        _Amtr_Test,             // (72) // 0x47
        _Htemp_Threshold,       // (73) // 0x48
        _Ftemp_Threshold,       // (74) // 0x49
        __HF1_Po_Max,           // (75) // 0x4A
        __HF1_Po_Mid2,          // (76) // 0x4B
        __HF1_Po_Mid1,          // (77) // 0x4C
        __HF1_Po_Min,           // (78) // 0x4D
        __HF2_Po_Max,           // (79) // 0x4E
        __HF2_Po_Mid2,          // (80) // 0x4F
        __HF2_Po_Mid1,          // (81) // 0x50
        __HF2_Po_Min,           // (82) // 0x51
        __HF3_Po_Max,           // (83) // 0x52
        __HF3_Po_Mid2,          // (84) // 0x53
        __HF3_Po_Mid1,          // (85) // 0x54
        __HF3_Po_Min,           // (86) // 0x55
        __50m_Po_Max,           // (87) // 0x56
        __50m_Po_Mid2,          // (88) // 0x57
        __50m_Po_Mid1,          // (89) // 0x58
        __50m_Po_Min,           // (90) // 0x59
        __VHF_Po_Max,           // (91) // 0x5A
        __VHF_Po_Mid,           // (92) // 0x5B
        __VHF_Po_Min,           // (93) // 0x5C
        __UHF_Po_Max,           // (94) // 0x5D
        __UHF_Po_Min,           // (95) // 0x5E
        __unknown1,             // (96) // 0x5F
        __unknown2,             // (97) // 0x60
        ;
        private final int     min, max;

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         */
        private Field(){
            this( false );
        } // Ft897calibration.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  signed                      whether the value is signed.
         */
        private Field( boolean signed ){
            min = signed ? Byte.MIN_VALUE : 0;
            max = Byte.MAX_VALUE - Byte.MIN_VALUE + min;
        } // Ft897calibration.Field constructor

        //------------------------------------------------------------------------------------------
        /** get retrieves the field from a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @return Object                      - the field.
         */
        public Object get( byte[] buffer ){
            return min < 0 ? buffer[ ordinal() ] : 0xFF & buffer[ ordinal() ];
        } // Ft897calibration.Field.get

        //------------------------------------------------------------------------------------------
        /** put places a field into a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @param  object                      the field.
         */
        public void put( byte[] buffer, Object object ){
            buffer[ ordinal() ] = (byte) FtRemote.putValue( 0, 1, min, max, null, object );
        } // Ft897calibration.Field.put

        //------------------------------------------------------------------------------------------
        /** type returns the Type of the field.
         *
         *  @return Type                        - the Type of field.
         */
        public Type type(){
            return min < 0 ? Type.SIGNED : Type.OTHER;
        } // Ft897calibration.Field.type

    } // Ft897calibration.Field enum

    static private final long    serialVersionUID = 2006040900L;
    static public  final int[]   addresses        = { 0x0007, 0x0051 },
                                 limits           = { 74, 97 };
    static private final short[] unknown_0x0051   = { 0xA1, 0x69, 0x21, 0x10,
                                                      0x9C, 0x65, 0x20, 0x0F,
                                                      0x9A, 0x65, 0x1F, 0x0F,
                                                      0x91, 0x5E, 0x32, 0x0C,
                                                      0x5B, 0x30, 0x0B,
                                                      0x75, 0x11,
                                                      0xBF, 0xA0 };
           private       byte[]  buffer           = new byte[ limits[ limits.length - 1 ] ];

    //----------------------------------------------------------------------------------------------
    /** Ft897calibration constructor.
     *
     *  @param  radio                           the radio's buffer.
     */
    Ft897calibration( byte[] radio ){
        int address = addresses[ 0 ];
        for ( int index = 0;  index < buffer.length;  buffer[ index++ ] = radio[ address++ ] );
    } // Ft897calibration constructor

    //----------------------------------------------------------------------------------------------
    /** getFields returns an array of the fields.  Overrides Ft___Settings.
     *
     *  @return FtField[]                       - the array of Fields.
     */
    FtField[] getFields(){
        return Field.values();
    } // Ft897calibration.getFields

    //----------------------------------------------------------------------------------------------
    /** getField returns a Field by name.  Overrides Ft___Settings.
     *
     *  @param  name                            the field name.
     *  @return FtField                         - the Field.
     */
    FtField getField( String name ){
        return Field.valueOf( name );
    } // Ft897calibration.getField

    //----------------------------------------------------------------------------------------------
    /** set updates the radio buffer from the internal buffer.  Overrides Ft___Calibration.
     *
     *  @param  radio                           the radio's buffer.
     */
    void set( byte[] radio ){
        int address = addresses[ 0 ];
        for ( int index = 0;  index < buffer.length;  radio[ address++ ] = buffer[ index++ ] );
    } // Ft897calibration.set
    
    //----------------------------------------------------------------------------------------------
    /** getBuffer returns the data buffer.  Overrides Ft___Settings.
     *
     *  @return byte[]                          - the buffer.
     */
    public byte[] getBuffer(){
        return buffer;
    } // Ft897channel.getBuffer

} // class Ft897calibration
