package net.ae7q.memory;

//==================================================================================================
/** Ft897calibration implements the options data for an FT-897.
 *
 *  @version    2006.04.09
 *  @author     Copyright © 2006 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
class Ft897options extends Ft___Options {

    //==============================================================================================
    /** Subbands describes the radio's subbands.
     */
    public enum Subbands  implements FtEnumPrefix {
        _160m (   1800,   2000 ),
        _80m  (   3500,   4000 ),
        _60m  (   5255,   5405 ),
        _40m  (   7000,   7300 ),
        _30m  (  10100,  10150 ),
        _20m  (  14000,  14350 ),
        _17m  (  18068,  18168 ),
        _15m  (  21000,  21450 ),
        _12m  (  24890,  24990 ),
        _10m  (  28000,  29700 ),
        _6m   (  33000,  56000 ),
        _FM   (  76000, 108000 ),
        _AIR  ( 118000, 137000 ),
        _2m   ( 137000, 164000 ),
        _70cm ( 420000, 470000 ),
        _WIDE (    100,  33000 ),
        ;
        int lower, upper;

        //------------------------------------------------------------------------------------------
        /** Subbands constructor.
         *
         *  @param  lower                       the lower frequency limit in kHz.
         *  @param  upper                       the upper frequency limit in kHz.
         */
        Subbands( int lower, int upper ){
            this.lower = lower;
            this.upper = upper;
        } // Ft897options.Subbands constructor
    } // Ft897options.Subbands enum

    //==============================================================================================
    /** Field describes all the radio's options.
     */
    public enum Field implements FtField, FtEnumPrefix {
        _Extended_Menu     ( 41, 0x80, FtSettings.Booleans.values() ),                  // Menu:01
        _144MHz_Ars        ( 10, 0x40, FtSettings.Booleans.values() ),                  // Menu:02
        _430MHz_Ars        ( 10, 0x20, FtSettings.Booleans.values() ),                  // Menu:03
        _AM_FM_Dial        ( 15, 0x80, new String[]{ "ENABLE", "DISABLE" } ),           // Menu:04
        _AM_Mic_Gain_Pct   ( 19, 0x7F,   0,   1,    0, 100 ),                           // Menu:05
//      _AM_Step           ----- In channel memory -----                                // Menu:06
        _APO_Time_Hrs      ( 16, 0x07,   0,   1,    1,   6, new String[]{ "OFF" } ),    // Menu:07
        _ARTS_Beep         (  8, 0xC0, new String[]{ "OFF", "RANGE", "ALL" } ),         // Menu:08
        _ARTS_ID           (  8, 0x10, FtSettings.Booleans.values() ),                  // Menu:09
        _ARTS_ID_Text      ( 70, 0,      0,  10, morse ),                               // Menu:10
        _Beacon_Play1      ( 80, 0,      0,  40, morse ),                               // Menu:11-1
        _Beacon_Play2      (120, 0,      0,  40, morse ),                               // Menu:11-2
        _Beacon_Play3      (160, 0,      0,  40, morse ),                               // Menu:11-3
        _Beacon_Time_Secs  ( 62, 0xFF,   0,   1,    1, 255, new String[]{ "OFF" } ),    // Menu:12
        _Beep_Tone         (  7, 0x03, new String[]{ "440Hz", "880Hz", "1760Hz" } ),    // Menu:13
        _Beep_Volume_Pct   ( 39, 0x7F,   0,   1,    0, 100 ),                           // Menu:14
        _Carrier_LSB_Rx_Hz ( 28, 0xFF,   0,  10,  -30,  30 ),                           // Menu:15
        _Carrier_LSB_Tx_Hz ( 30, 0xFF,   0,  10,  -30,  30 ),                           // Menu:16
        _Carrier_USB_Rx_Hz ( 29, 0xFF,   0,  10,  -30,  30 ),                           // Menu:17
        _Carrier_USB_Tx_Hz ( 31, 0xFF,   0,  10,  -30,  30 ),                           // Menu:18
        _Cat_Rate          ( 44, 0xC0, new String[]{ "4800", "9600", "38400" } ),       // Menu:19
        _Cat_Lin_Tun       ( 38, 0x0C, new String[]{ "CAT", "LINEAR", "TUNER" } ),      // Menu:20
        _Clarifier_Knob    (  7, 0x0C, new String[]{ "MAIN", "VFO/MEM", "CLAR" } ),     // Menu:21
        _CW_Auto           ( 42, 0x80, FtSettings.Booleans.values() ),                  // Menu:22
        _CW_BFO            ( 42, 0x03, new String[]{ "USB", "LSB", "AUTO" } ),          // Menu:23
        _CW_Delay_Ms       ( 11, 0xFF80, 0,  10,    3, 300, new String[]{ "FULL" },     // Menu:24
                                                            true ),
        _CW_Key_Rev        (  3, 0x04, new String[]{ "NORMAL", "REVERSE" } ),           // Menu:25
        _CW_Mic_Key        ( 19, 0x80, FtSettings.Booleans.values() ),                  // Menu:26
        _CW_Pitch_Hz       (  9, 0x07, 400, 100,    0,   4 ),                           // Menu:27
        _CW_QSK_Ms         ( 42, 0x1C,  10,   5,    0,   4 ),                           // Menu:28
        _CW_Side_Tone_Pct  ( 12, 0x7F,   0,   1,    0, 100 ),                           // Menu:29
        _CW_Speed          ( 13, 0x3F,   4,   1,    0,  56 ),                           // Menu:30
        _CW_Speed_Units    (  3, 0x08, new String[]{ "WPM", "CPM" } ),                  // Menu:30
        _CW_Train          ( 42, 0x60, new String[]{ "N:", "A:", "AN:" } ),             // Menu:31
        _CW_Weight         ( 10, 0x1F, new String[]{ "2.5", "2.6", "2.7", "2.8", "2.9", // Menu:32
                                                     "3.0", "3.1", "3.2", "3.3", "3.4",
                                                     "3.5", "3.6", "3.7", "3.8", "3.9",
                                                     "4.0", "4.1", "4.2", "4.3", "4.4", "4.5" } ),
//      _DCS_Code          ----- In channel memory -----                                // Menu:33
        _DCS_Inv           ( 17, 0xC0, new String[]{ "TnRn","TnRiv","TivRn","TivRiv"} ),// Menu:34
        _Dial_Step         (  8, 0x20, new String[]{ "FINE", "COARSE" } ),              // Menu:35
        _Dig_Disp_Hz       ( 26, 0xFFFF, 0,  10, -300, 300 ),                           // Menu:36
        _Dig_Gain_Pct      ( 21, 0x7F,   0,   1,    0, 100 ),                           // Menu:37
        _Dig_Mode          ( 16, 0xE0, new String[]{ "RTTY-L", "RTTY-U", "PSK31-L",     // Menu:38
                                                     "PSK31-U", "USER-L",  "USER-U" } ),
        _Dig_Shift_Hz      ( 24, 0xFFFF, 0,  10, -300, 300 ),                           // Menu:39
        _Dig_VOX_Pct       ( 40, 0x7F,   0,   1,    0, 100 ),                           // Menu:40
        _Disp_Color        ( 46, 0x07, new String[]{ "ARTS", "BAND", "FIX", "MEMGRP",   // Menu:41
                                                     "MODE", "MTR", "VFO" } ),
        _Disp_Color_ARTS   ( 47, 0x01,   1,   1 ),                                      // Menu:41
        _Disp_Color_Band   ( 47, 0x02,   1,   1 ),                                      // Menu:41
        _Disp_Color_MemGrp ( 47, 0x08,   1,   1 ),                                      // Menu:41
        _Disp_Color_Mode   ( 47, 0x10,   1,   1 ),                                      // Menu:41
        _Disp_Color_Mtr    ( 47, 0x20,   1,   1 ),                                      // Menu:41
        _Disp_Color_VFO    ( 47, 0x40,   1,   1 ),                                      // Menu:41
        _Disp_Color_Fix    ( 48, 0x1F,   1,   1 ),                                      // Menu:41
        _Disp_Contrast     (  6, 0x0F,  -2,   1,    3,  15 ),                           // Menu:42
        _Disp_Intensity    ( 46, 0x30,   0,   1 ),                                      // Menu:43
        _Disp_Mode         (  6, 0xC0, new String[]{ "OFF", "AUTO1", "AUTO2", "ON" } ), // Menu:44
        _DSP_BPF           ( 43, 0x0C, new String[]{ "60Hz", "120Hz", "240Hz" } ),      // Menu:45
        _DSP_HPF           ( 45, 0x0F, new String[]{ "100Hz", "160Hz", "220Hz", "280Hz",// Menu:46
                                                     "340Hz", "400Hz", "460Hz", "520Hz",
                                                     "580Hz", "640Hz", "720Hz", "760Hz",
                                                     "820Hz", "880Hz", "940Hz", "1000Hz" } ),
        _DSP_LPF           ( 44, 0x1F, new String[]{"1000Hz","1160Hz","1320Hz","1480Hz",// Menu:47
                                                    "1650Hz","1800Hz","1970Hz","2130Hz",
                                                    "2290Hz","2450Hz","2610Hz","2770Hz",
                                                    "2940Hz","3100Hz","3260Hz","3420Hz",
                                                    "3580Hz","3740Hz","3900Hz","4060Hz",
                                                    "4230Hz","4390Hz","4550Hz","4710Hz",
                                                    "4870Hz","5030Hz","5190Hz","5390Hz",
                                                    "5520Hz","5680Hz","5840Hz","6000Hz" } ),
        _Disp_Mic_Eq       ( 43, 0x03, new String[]{ "OFF", "LPF", "HPF", "BOTH" } ),   // Menu:48
        _DSP_NR            ( 43, 0xF0,   1,   1 ),                                      // Menu:49
        _Emergency_Alaska  ( 15, 0x20, FtSettings.Booleans.values() ),                  // Menu:50
        _FM_Mic_Gain_Pct   ( 20, 0x7F,   0,   1,    0, 100 ),                           // Menu:51
//      _FM_Step           ----- In channel memory -----                                // Menu:52
        _Home_To_VFO       ( 41, 0x40, FtSettings.Booleans.values() ),                  // Menu:53
        _Lock_Mode         (  9, 0x30, new String[]{ "DIAL", "FREQ", "PANEL", "ALL"} ), // Menu:54
        _Mem_Grouping      ( 16, 0x10, FtSettings.Booleans.values() ),                  // Menu:55
//      _Memory_Tag        ----- In channel memory -----                                // Menu:56
        _MemVFO_Knob_Mode  ( 52, 0x07, new String[]{ "CW SIDETONE", "CW SPEED",         // Menu:57
                                                     "MHz/MEM GRP", "MIC GAIN",
                                                     "NB LEVEL",    "RF POWER", "STEP" } ),
        _Mic_Scan          ( 18, 0x80, FtSettings.Booleans.values() ),                  // Menu:58
        _Mic_Sel           ( 38, 0x30, new String[]{ "NOR", "RMT", "CAT" } ),           // Menu:59
        _Mtr_ARx_Sel       ( 61, 0x07, new String[]{ "SIG", "CTR", "VLT", "N/A",        // Menu:60
                                                     "FS",  "OFF" } ),
        _Mtr_ATx_Sel       ( 45, 0xE0, new String[]{ "PWR", "ALC", "MOD", "SWR",        // Menu:61
                                                     "VLT", "N/A", "OFF" } ),
        _Mtr_Peak_Hold     ( 38, 0x40, FtSettings.Booleans.values() ),                  // Menu:62
        _NB_Level_Pct      ( 49, 0x7F,   0,   1,    0, 100 ),                           // Menu:63
        _Op_Filter_Name1   (208, 0, 0,   4, morse ),                                    // Menu:64-1
        _Op_Filter_Name2   (212, 0, 0,   4, morse ),                                    // Menu:64-2
        _Prog_A            ( 53, 0xFF, programmableOptions ),                           // Menu:65
        _Prog_B            ( 54, 0xFF, programmableOptions ),                           // Menu:66
        _Prog_C            ( 55, 0xFF, programmableOptions ),                           // Menu:67
        _Prog_Acc          ( 56, 0xFF, programmableOptions ),                           // Menu:68
        _Prog_P1           ( 57, 0xFF, programmableOptions ),                           // Menu:69
        _Prog_P2           ( 58, 0xFF, programmableOptions ),                           // Menu:70
        _Pkt_1200_Gain_Pct ( 22, 0x7F,   0,   1,    0, 100 ),                           // Menu:71
        _Pkt_9600_Gain_Pct ( 23, 0x7F,   0,   1,    0, 100 ),                           // Menu:72
        _Pkt_Rate          (  8, 0x04, new String[]{ "1200bps", "9600bps" } ),          // Menu:73
        _Proc_Level_Pct    ( 50, 0x7F,   0,   1,    0, 100 ),                           // Menu:74
        _RF_Power_HF_Watts ( 51, 0x7F,   0,   1,    5, 100 ),                           // Menu:75
        _RF_Power_6M_Watts ( 66, 0x7F,   0,   1,    5, 100 ),                           // Menu:75
        _RF_Power_VHF_Watts( 67, 0x7F,   0,   1,    5,  50 ),                           // Menu:75
        _RF_Power_UHF_Watts( 68, 0x7F,   0,   1,    2,  20 ),                           // Menu:75
//      _Split_Tx_Freq_Hz  ----- In channel memory -----                                // Menu:76
        _Scan_Mode         ( 41, 0x30, new String[]{ "TIME", "BUSY", "STOP" } ),        // Menu:77
        _Scan_Resume_Secs  ( 41, 0x0F,   0,   1,    1,  10 ),                           // Menu:78
//      _Tone_Split        ----- In channel memory -----                                // Menu:79
        _Squelch_Knob      ( 10, 0x80, new String[]{ "RF-GAIN", "SQL" } ),              // Menu:80
        _SSB_Mic_Gain_Pct  ( 18, 0x7F,   0,   1,    0, 100 ),                           // Menu:81
//      _SSB_Step          ----- In channel memory -----                                // Menu:82
//      _Tone_CTCSS        ----- In channel memory -----                                // Menu:83
        _Tx_Time_Mins      ( 17, 0x1F,   0,   1,    1,  20, new String[]{ "OFF" } ),    // Menu:84
        _Tune_Mode         ( 52, 0x38, new String[]{ "OFF", "ATAS(HF)", "ATAS(HF&50)",  // Menu:85
                                                     "ATAS(ALL)", "TUNER" } ),
        _Tx_IF_Filter      ( 60, 0x03, filters ),                                       // Menu:86
        _VOX_Delay_Ms      ( 15, 0x1F,   0, 100,    1,  30 ),                           // Menu:87
        _VOX_Gain_Pct      ( 14, 0x7F,   0,   1,    1, 100 ),                           // Menu:88
        _Xvtr_A_Negative   ( 61, 0x10, FtSettings.Booleans.values() ),                  // Menu:89
        _Xvtr_A_Offset_Hz  (200, 0xFFFFFFFFL, 0, 10 ),                                  // Menu:89
        _Xvtr_B_Negative   ( 61, 0x08, FtSettings.Booleans.values() ),                  // Menu:90
        _Xvtr_B_Offset_Hz  (204, 0xFFFFFFFFL, 0, 10 ),                                  // Menu:90
        _Xvtr_Sel          ( 59, 0x18, new String[]{ "OFF", "XVTR A", "XVTR B" } ),     // Menu:91
    //  ----- Front panel settings ----- 
        _Current_Option_Menu_Index   ( 32, 0x7F, menuOptions ),                         // Knob
        _Home              (  0, 0x10, FtSettings.Booleans.values() ),                  // HOME
        _M_Tune            (  0, 0x20, FtSettings.Booleans.values() ),                  // Dial
        _MemVFO            (  0, 0xC0, new String[]{ null, "MEMORY", "VFO" } ),         // V/M
        _VFO_A_Subband     (  4, 0x0F, Subbands.values() ),                             // Band
        _VFO_B_Subband     (  4, 0xF0, Subbands.values() ),                             // Band
        _IF_Shift          (  2, 0x10, FtSettings.Booleans.values() ),                  // IF shift
        _Unlocked          (  2, 0x40, FtSettings.Booleans.values() ),                  // Lock
        _Slow_Tune_Rx      (  2, 0x80, FtSettings.Booleans.values() ),                  // Fast
    //  ----- MH59 buttons ----- 
        _MH59_Dial         ( 37, 0x01, FtSettings.Booleans.values() ),                  // MH59dial
        _MH59_AF_Gain      ( 37, 0x02, FtSettings.Booleans.values() ),                  // MH59gain
        _Current_Memory_Channel_Index( 216, 0xFF, 1, 1,    0, 215 ),                    // Knob
    //  ----- MF settings ----- 
        _Current_Function_Row_Index  ( 33, 0x0F ),                                      // Knob
        _VFO_B             (  0, 0x01, FtSettings.Booleans.values() ),                  // MFa:A
//      _VFO_Copy          ----- Momentary -----                                        // MFa:B
        _Split_VFO         ( 37, 0x80, FtSettings.Booleans.values() ),                  // MFa:C
//      _Visible          ----- In channel memory -----                                 // MFb:AB
//      _Skip             ----- In channel memory -----                                 // MFb:B
//      _Display_Tag       ----- In channel memory -----                                // MFb:C
//      _Memory_Store      ----- Momentary -----                                        // MFc:A
        _QMB               (  0, 0x04, FtSettings.Booleans.values() ),                  // MFc:B
        _MT_QMB            (  0, 0x02, FtSettings.Booleans.values() ),                  // MFc:B
        _Speech_Processor  ( 65, 0x02, FtSettings.Booleans.values() ),                  // MFc:C
//      _Split_Tx          ----- In channel memory -----                                // MFd:A
//      _Reverse           ----- Unavailable -----                                      // MFd:B
        _VOX               (  3, 0x80, FtSettings.Booleans.values() ),                  // MFd:C
//      _Encode            ----- In channel memory -----                                // MFe:A
//      _Decode            ----- In channel memory -----                                // MFe:B
//      _Tone_Search       ----- Momentary -----                                        // MFe:C
        _Arts              ( 36, 0x80, FtSettings.Booleans.values() ),                  // MFf:A
//      _Band_Search       ----- Momentary -----                                        // MFf:B
        _PMS_Tune_Limits   (  0, 0x08, FtSettings.Booleans.values() ),                  // MFf:C
        _Memory_Scan       ( 36, 0x60, new String[]{"OFF",null,"SCAN UP","SCAN DOWN"} ),// MFg:A
        _Dual_Scan         ( 36, 0x18, new String[]{"OFF",null,"PRIORITY","WATCH"} ),   // MFg:BC
//      _Scope             ----- Unavailable -----                                      // MFh:A
        _Scope_Channels    ( 65, 0x0C, new String[]{"21", "31", "127"} ),               // MFh:B
        _Scope_One_Sweep   (  8, 0x08, FtSettings.Booleans.values() ),                  // MFh:B
        _Scope_Peak_Hold   ( 65, 0x10, FtSettings.Booleans.values() ),                  // MFh:C
//      _Scope_Step        ----- Unavailable -----                                      // MFh:C
        _Meter_Mode        (  3, 0x03, new String[]{ "PWR", "ALC", "SWR", "MOD" } ),    // MFi:AB
        _Display_Large     (  2, 0x04, FtSettings.Booleans.values() ),                  // MFi:C
//      _CW_Spot           ----- Momentary -----                                        // MFj:A
        _CW_Break          (  3, 0x20, FtSettings.Booleans.values() ),                  // MFj:B
        _CW_Keyer          (  3, 0x10, FtSettings.Booleans.values() ),                  // MFj:C
        _Tuner             ( 69, 0x80, FtSettings.Booleans.values() ),                  // MFk:A
//      _Tune_Down         ----- Momentary -----                                        // MFk:B
//      _Tune_Up           ----- Momentary -----                                        // MFk:C
        _Noise_Blanker     (  2, 0x20, FtSettings.Booleans.values() ),                  // MFl:A
        _AGC               ( 64, 0x20, FtSettings.Booleans.values() ),                  // MFl:B
        _AGC_Mode          (  2, 0x03, new String[]{ "AUTO", "FAST", "SLOW" } ),        // MFl:C
//      _Rf_Gain           ----- In channel memory -----                                // MFm:AB
//      _Narrow_Band_FM    ----- In channel memory -----                                // MFm:C
        _Rx_Dig_Filter     ( 63, 0x30, filters ),                                       // MFn:ABC
        _Rx_SSB_Filter     ( 63, 0x03, filters ),                                       // MFn:ABC
        _Rx_CW_Filter      ( 63, 0x0C, filters ),                                       // MFn:ABC
//      _Play1             ----- Momentary -----                                        // MFo:A
//      _Play2             ----- Momentary -----                                        // MFo:B
//      _Play3             ----- Momentary -----                                        // MFo:C
        _DSP_DNR           ( 64, 0x02, FtSettings.Booleans.values() ),                  // MFp:A
        _DSP_DNF           ( 64, 0x01, FtSettings.Booleans.values() ),                  // MFp:B
        _DSP_DBF           ( 64, 0x0C, new String[]{ "OFF", null, null, "ON" } ),       // MFp:C
    //  ----- Automatic settings ----- 
        __rfPowerHFunknown (51, 0x80, FtSettings.Booleans.values() ),                   //(auto)
        __rfPower6Munknown (66, 0x80, FtSettings.Booleans.values() ),                   //(auto)
        __rfPowerVHFunknown(67, 0x80, FtSettings.Booleans.values() ),                   //(auto)
        __rfPowerUHFunknown(68, 0x80, FtSettings.Booleans.values() ),                   //(auto)
//      __DigVox           ( 65, 0x80, FtSettings.Booleans.values() ),                  //(auto)
//      __HomeBand         (  1, 0x03, HomeBands.values() ),                            //(auto)
//      __IfShiftPriority  (  2, 0x08 ),                                                // IF shift
//      __rxNarrowFilter   ( 63, 0x80, FtSettings.Booleans.values() ),                  //(auto)
    //  ----- Unknown settings ----- 
        __unknown01a       (  1, 0x7C ),     // 0x00
        __unknown01b       (  1, 0x80 ),     // ??? = 0x01
//      __unknown03        (  3, 0x40 ),     // 0x00  -- FT-817 ??
//      __unknown06        (  6, 0x30 ),     // 0x00  -- FT-817 ??
//      __unknown07        (  7, 0xF0 ),     // 0x00  -- FT-817 ??
//      __unknown08        (  8, 0x03 ),     // 0x00  -- FT-817 ??
//      __unknown09a       (  9, 0x08 ),     // 0x00  -- FT-817 ??
//      __unknown09b       (  9, 0xC0 ),     // 0x00  -- FT-817 ??
        __unknown13        ( 13, 0xC0 ),     // 0x01  -- FT-817 ??
//      __unknown15        ( 15, 0xC0 ),     // 0x00  -- FT-817 ??
        __unknown16        ( 16, 0x08 ),     // 0x00
        __unknown17        ( 17, 0x20 ),     // 0x00
        __unknown34        ( 34, 0xFFFF ),   // 0x00
//      __unknown36a       ( 36, 0x07 ),     // 0x00  -- FT-817 ??
//      __unknown37        ( 37, 0x7C ),     // 0x00  -- FT-817 ??
//      __unknown38a       ( 38, 0x03 ),     // 0x00  -- FT-817 ??
        __unknown38b       ( 38, 0x80 ),     // 0x00
        __unknown44        ( 44, 0x20 ),     // 0x00
        __unknown45        ( 45, 0x10 ),     // ??? = 0x01
        __unknown46a       ( 46, 0x08 ),     // 0x00
        __unknown46b       ( 46, 0xC0 ),     // 0x00
        __unknown47a       ( 47, 0x04 ),     // 0x00
        __unknown47b       ( 47, 0x80 ),     // 0x00
        __unknown48        ( 48, 0xE0 ),     // 0x00
        __unknown52        ( 52, 0xC0 ),     // 0x00
        __unknown59a       ( 59, 0x07 ),     // 0x00
        __unknown59b       ( 59, 0xE0 ),     // ??? = 0x03
        __unknown60        ( 60, 0xFC ),     // ??? = 0x01
        __unknown61        ( 61, 0xE0 ),     // ??? = 0x03
        __unknown63        ( 63, 0x40 ),     // 0x00
        __unknown64a       ( 64, 0x10 ),     // 0x00
        __unknown64b       ( 64, 0xC0 ),     // 0x00
        __unknown65a       ( 65, 0x01 ),     // 0x00
        __unknown65b       ( 65, 0x60 ),     // 0x00
        __unknown69        ( 69, 0x7F ),     // ??? = 0x28
        ;
        private final boolean  special;
        private final Object[] keys;
        private final long     mask, base, min, max;
        private final int      offset, shift, width, scale, length;

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         */
        private Field( int offset, long mask ){
            this( offset, mask, 0, 1, 0, -1, null, false );
        } // Ft817options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  keys                        the field's keys.
         */
        private Field( int offset, long mask, Object[] keys ){
            this( offset, mask, 0, 1, 0, keys.length - 1, keys, false );
        } // Ft897options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         */
        private Field( int offset, long mask, long base, int scale ){
            this( offset, mask, base, scale, 0, -1, null, false );
        } // Ft897options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  keys                        the field's keys.
         */
        private Field( int offset, long mask, long base, int scale, Object[] keys ){
            this( offset, mask, base, scale, 0, keys.length - 1, keys, false );
        } // Ft897options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  min                         the field's minimum value.
         *  @param  max                         the field's maximum value.
         */
        private Field( int offset, long mask, long base, int scale, long min, long max ){
            this( offset, mask, base, scale, min, max, null, false );
        } // Ft897options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  min                         the field's minimum value.
         *  @param  max                         the field's maximum value.
         *  @param  keys                        the field's keys.
         */
        private Field( int offset, long mask, long base, int scale, long min, long max,
                       Object[] keys ){
            this( offset, mask, base, scale, min, max, keys, false );
        } // Ft897options.Field constructor

        //------------------------------------------------------------------------------------------
        /** Field constructor.
         *
         *  @param  offset                      the field's offset.
         *  @param  mask                        the field's mask.
         *  @param  base                        the field's base.
         *  @param  scale                       the field's scale.
         *  @param  min                         the field's minimum value.
         *  @param  max                         the field's maximum value.
         *  @param  keys                        the field's keys.
         *  @param  special                     the field's special flag.
         */
        private Field( int offset, long mask, long base, int scale, long min, long max,
                       Object[] keys, boolean special ){
            int shift, width;
            this.offset  = offset;
            this.mask    = mask;
            for ( shift = 0;  mask != 0  && (mask & 1) == 0;  shift++, mask >>>= 1 );
            for ( width = 0;  mask != 0;                      width++, mask >>>= 1 );
            this.shift   = shift;
            this.width   = width;
            this.length  = this.mask != 0 ? (shift + width - 1) / 8 + 1 : scale;
            this.base    = base;
            this.scale   = scale;
            this.min     = min;
            this.max     = max < min ? this.mask >>> shift : max;
            this.special = special;
            this.keys    = keys;
        } // Ft897options.Field constructor

        //------------------------------------------------------------------------------------------
        /** get retrieves the field from a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @return Object                      - the field.
         */
        public Object get( byte[] buffer ){
            if ( mask == 0 ){
                char[] temp = new char[ length ];
                for ( int index = 0;  index < length;  index++ ){
                    temp[ index ] = keys[ 0 ].toString().charAt( buffer[ offset + index ] );
                }
                return new String( temp );
            } else {
                long value = 0;
                for ( int index = 0;  index < length;  index++  ){
                    value = value << 8 | buffer[ offset + index ] & 0xFF;
                }
                value = (mask & value) >>> shift;
                if ( special ){
                    value = value >> 1 | value << 8 & 0x100;
                }
                return FtRemote.getValue( value, base, scale, min, width, keys );
            }
        } // Ft897options.Field.get

        //------------------------------------------------------------------------------------------
        /** put places a field into a buffer.
         *
         *  @param  buffer                      the buffer.
         *  @param  object                      the field.
         */
        public void put( byte[] buffer, Object object ){
            long value;
            if ( mask == 0 ){
                String string = (String) object;
                String morse  = keys[ 0 ].toString();
                for ( int index = 0;  index < length;  index++ ){
                    value = index < string.length() ? morse.indexOf( string.charAt( index ) ) : -1;
                    buffer[ offset + index ] = (byte) (value < 0 ? morse.indexOf( ' ' ) : value);
                }
            } else {
                long mask = this.mask;
                value = FtRemote.putValue( base, scale, min, max, keys, object );
                if ( special ){
                    buffer[ offset ] = (byte) value;
                    buffer[ offset + 1 ] ^= mask & (buffer[ offset + 1 ] ^ value >> 1);
                } else {
                    value <<= shift;
                    for ( int index = length;  index-- > 0;  value >>>= 8, mask >>>= 8 ){
                        buffer[ offset + index ] ^= mask & (buffer[ offset + index ] ^ value);
                    }
                }
            }
        } // Ft897options.Field.put

        //------------------------------------------------------------------------------------------
        /** type returns the Type of the field.
         *
         *  @return Type                        - the Type of field.
         */
        public Type type(){
            return mask == 0 ? Type.MORSE : Type.OTHER;
        } // Ft897options.Field.type

    } // Ft897options.Field enum

    static private final long     serialVersionUID = 2006040900L;
    static private final int[]    addresses        = { 0x0068, 0x1C59, 0x050E },
                                  limits           = {     70,    216,    217 };
    static private final String[] functions
        = { "MFa:A/B",     "MFa:A=B",     "MFa:SPL",    "MFb:MW",   "MFb:SKIP/MCLR", "MFb:TAG",
            "MFc:STO",     "MFc:RCL",     "MFc:PROC",   "MFd:RPT",  "MFd:REV",       "MFd:VOX",
            "MFe:TON/ENC", "MFe:TON/DEC", "MFe:TDCH",   "MFf:ARTS", "MFf:SRCH",      "MFf:PMS",
            "MFg:SCN",     "MFg:PRI",     "MFg:DW",     "MFh:SCOP", "MFh:WID",       "MFh:STEP",
            "MFi:MTR",     "MFi:SWR",     "MFi:DISP",   "MFj:SPOT", "MFj:BK",        "MFj:KYR",
            "MFk:TUNE",    "MFk:DOWN",    "MFk:UP",     "MFl:NB",   "MFl:AGC",       "MFl:AGC SEL",
            "MFm:IPO",     "MFm:ATT",     "MFm:NAR",    "MFn:CFIL", "MFn:FIL1",      "MFn:FIL2",
            "MFo:PLY1",    "MFo:PLY2",    "MFo:PLY3",   "MFp:DNR",  "MFp:DNF",       "MFp:DBF" };
    static private final String[] menuOptions
        = { "01:EXT MENU",          "02:144MHz ARS",     "03:430MHz ARS",     "04:AM&FM DIAL",
            "05:AM MIC GAIN",       "06:AM STEP",        "07:APO TIME",       "08:ARTS BEEP",
            "09:ARTS ID",           "10:ARTS IDW",       "11:BEACON TEXT",    "12:BEACON TIME",
            "13:BEEP TONE",         "14:BEEP VOL",       "15:CAR LSB R",      "16:CAR LSB T",
            "17:CAR USB R",         "18:CAR USB T",      "19:CAT RATE",       "20:CAT/LIN/TUN",
            "21:CLAR DIAL SEL",     "22:CW AUTO MODE",   "23:CW BFO",         "24:CW DELAY",
            "25:CW:KEY REV",        "26:CW PADDLE",      "27:CW PITCH",       "28:CW QSK",
            "29:CW SIDE TONE",      "30:CW:SPEED",       "31:CW TRAINING",    "32:CW WEIGHT",
            "33:DCS CODE",          "34:DCS INV",        "35:DIAL STEP",      "36:DIG DISP",
            "37:DIG GAIN",          "38:DIG MODE",       "39:DIG SHIFT",      "40:DIG VOX",
            "41:DISP COLOR",        "42:DISP CONTRAST",  "43:DISP INTENSITY", "44:DISP MODE",
            "45:DSP BPF WIDTH",     "46:DSP HPF CUTOFF", "47:DSP LPF CUTOFF", "48:DSP MIC EQ",
            "49:DSP NR LEVEL",      "50:EMERGENCY",      "51:FM MIC GAIN",    "52:FM STEP",
            "53:HOME->VFO",         "54:LOCK MODE",      "55:MEM GROUP",      "56:MEM TAG",
            "57:MEM/VFO DIAL MODE", "58:MIC SCAN",       "59:MIC SEL",        "60:MTR ARX",
            "61:MTR ATX",           "62:MTR PEAK HOLD",  "63:NB LEVEL",       "64:OP FILTER",
            "65:PG A",              "66:PG B",           "67:PG C",           "68:PG ACC",
            "69:PG P1",             "70:PG P2",          "71:PKT 1200",       "72:PKT 9600",
            "73:PKT RATE",          "74:PROC LEVEL",     "75:RF POWER SET",   "76:RPT SHIFT",
            "77:SCAN MODE",         "78:SCAN RESUME",    "79:SPLIT TONE",     "80:SQL/RF GAIN",
            "81:SSB MIC GAIN",      "82:SSB STEP",       "83:TONE FREQ",      "84:TX TIME",
            "85:TUNER/ATAS",        "86:TX IF FILTER",   "87:VOX DELAY",      "88:VOX GAIN",
            "89:XVTR A FREQ",       "90:XVTR B FREQ",    "91:XVTR SEL" };
    static private final String[] programmableOptions
        = { "MFa:A/B",     "MFa:A=B",     "MFa:SPL",    "MFb:MW",   "MFb:SKIP/MCLR", "MFb:TAG",
            "MFc:STO",     "MFc:RCL",     "MFc:PROC",   "MFd:RPT",  "MFd:REV",       "MFd:VOX",
            "MFe:TON/ENC", "MFe:TON/DEC", "MFe:TDCH",   "MFf:ARTS", "MFf:SRCH",      "MFf:PMS",
            "MFg:SCN",     "MFg:PRI",     "MFg:DW",     "MFh:SCOP", "MFh:WID",       "MFh:STEP",
            "MFi:MTR",     "MFi:SWR",     "MFi:DISP",   "MFj:SPOT", "MFj:BK",        "MFj:KYR",
            "MFk:TUNE",    "MFk:DOWN",    "MFk:UP",     "MFl:NB",   "MFl:AGC",       "MFl:AGC SEL",
            "MFm:IPO",     "MFm:ATT",     "MFm:NAR",    "MFn:CFIL", "MFn:FIL1",      "MFn:FIL2",
            "MFo:PLY1",    "MFo:PLY2",    "MFo:PLY3",   "MFp:DNR",  "MFp:DNF",       "MFp:DBF",
            "01:EXT MENU",          "02:144MHz ARS",     "03:430MHz ARS",     "04:AM&FM DIAL",
            "05:AM MIC GAIN",       "06:AM STEP",        "07:APO TIME",       "08:ARTS BEEP",
            "09:ARTS ID",           "10:ARTS IDW",       "11:BEACON TEXT",    "12:BEACON TIME",
            "13:BEEP TONE",         "14:BEEP VOL",       "15:CAR LSB R",      "16:CAR LSB T",
            "17:CAR USB R",         "18:CAR USB T",      "19:CAT RATE",       "20:CAT/LIN/TUN",
            "21:CLAR DIAL SEL",     "22:CW AUTO MODE",   "23:CW BFO",         "24:CW DELAY",
            "25:CW:KEY REV",        "26:CW PADDLE",      "27:CW PITCH",       "28:CW QSK",
            "29:CW SIDE TONE",      "30:CW:SPEED",       "31:CW TRAINING",    "32:CW WEIGHT",
            "33:DCS CODE",          "34:DCS INV",        "35:DIAL STEP",      "36:DIG DISP",
            "37:DIG GAIN",          "38:DIG MODE",       "39:DIG SHIFT",      "40:DIG VOX",
            "41:DISP COLOR",        "42:DISP CONTRAST",  "43:DISP INTENSITY", "44:DISP MODE",
            "45:DSP BPF WIDTH",     "46:DSP HPF CUTOFF", "47:DSP LPF CUTOFF", "48:DSP MIC EQ",
            "49:DSP NR LEVEL",      "50:EMERGENCY",      "51:FM MIC GAIN",    "52:FM STEP",
            "53:HOME->VFO",         "54:LOCK MODE",      "55:MEM GROUP",      "56:MEM TAG",
            "57:MEM/VFO DIAL MODE", "58:MIC SCAN",       "59:MIC SEL",        "60:MTR ARX",
            "61:MTR ATX",           "62:MTR PEAK HOLD",  "63:NB LEVEL",       "64:OP FILTER",
            "71:PKT 1200",          "72:PKT 9600",       "73:PKT RATE",       "74:PROC LEVEL",
            "75:RF POWER SET",      "76:RPT SHIFT",      "77:SCAN MODE",      "78:SCAN RESUME",
            "79:SPLIT TONE",        "80:SQL/RF GAIN",    "81:SSB MIC GAIN",   "82:SSB STEP",
            "83:TONE FREQ",         "84:TX TIME",        "85:TUNER/ATAS",     "86:TX IF FILTER",
            "87:VOX DELAY",         "88:VOX GAIN",       "89:XVTR A FREQ",    "90:XVTR B FREQ",
            "91:XVTR SEL",
            "MONI",  "Q:SPL",  "TCALL",  "ATC",  "USER" };
    static private final String[] filters  = { "CFIL", "FIL1", "FIL2" };
    static private final String[] morse    = { "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ /!+." };
           private       byte[]   buffer   = new byte[ limits[ limits.length - 1 ] ];

    //----------------------------------------------------------------------------------------------
    /** Ft897options constructor.
     *
     *  @param  radio                   the buffer containing the channel data.
     */
    Ft897options( byte[] radio ){
        for ( int offset = 0, index = 0;  index < addresses.length;  index++ ){
            for ( int address = addresses[ index ];  offset < limits[ index ];
                                                     buffer[ offset++ ] = radio[ address++ ] );
        }
    } // Ft897options constructor

    //----------------------------------------------------------------------------------------------
    /** getFields returns an array of the fields.  Overrides Ft___Settings.
     *
     *  @return FtField[]                       - the array of Fields.
     */
    FtField[] getFields(){
        return Field.values();
    } // Ft897options.getFields

    //----------------------------------------------------------------------------------------------
    /** getField returns a Field by name.  Overrides Ft___Settings.
     *
     *  @param  name                            the field name.
     *  @return FtField                         - the Field.
     */
    FtField getField( String name ){
        return Field.valueOf( name );
    } // Ft897options.getField

    //----------------------------------------------------------------------------------------------
    /** set updates the radio buffer from the internal buffer.  Overrides Ft___Options.
     *
     *  @param  radio                           the radio's buffer.
     */
    void set( byte[] radio ){
        for ( int offset = 0, index = 0;  index < addresses.length;  index++ ){
            for ( int address = addresses[ index ];  offset < limits[ index ];
                                                     radio[ address++ ] = buffer[ offset++ ] );
        }
    } // Ft897options.set
    
    //----------------------------------------------------------------------------------------------
    /** getBuffer returns the data buffer.  Overrides Ft___Settings.
     *
     *  @return byte[]                          - the buffer.
     */
    public byte[] getBuffer(){
        return buffer;
    } // Ft897channel.getBuffer

} // class Ft897options
