package net.ae7q.memory;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.logging.*;
import java.util.prefs.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
// import javax.comm.*;
import gnu.io.*;

import com.ultimeth.util.*;

//==================================================================================================
/** FTremote is a program for setting options in a Yaesu FT-897D.
 *
 *  @version    2006.04.09
 *  @author     Copyright © 2006 by Dean K. Gibson.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
class FtDisplay {
    static         final String  COPYRIGHT
                         = "Copyright \u00A9 2006-2010 by Dean K. Gibson" + Log.endOfLine;
    static private final String  PROGRAM_ID    = "FtRemote v0.04";
    static private final boolean LIST_UNKNOWNS = true;
                   final HashMap <KeyStroke, JComponent>
                                 keyStrokeMap  = new HashMap <KeyStroke, JComponent>();

                   final KeyAdapter keyAdapter = new KeyAdapter(){
                        public void keyPressed( KeyEvent event ){
                            Component  component = event.getComponent();
                            JComponent jComponent
                                = keyStrokeMap.get( KeyStroke.getKeyStrokeForEvent( event ) );
                            if ( jComponent instanceof JPopupMenu ){
                                 ((JPopupMenu) jComponent).show( component,
                                                                 component.getWidth()  / 2,
                                                                 component.getHeight() / 2 );
                            } else if ( jComponent instanceof JMenuItem ){
                                ((JMenuItem) jComponent).doClick();
                            } else {
                                jTabbedPane.setSelectedComponent( jComponent );
                            }
                        }
                    };// KeyAdapter subclass

                   final MouseAdapter mouseAdapter = new MouseAdapter(){
                        public void mousePressed( MouseEvent event ){
                            if ( event.isPopupTrigger() ){      // for Solaris
                                jPopupMenu.show( event.getComponent(), event.getX(), event.getY() );
                            }
                        }
                        public void mouseReleased( MouseEvent event ){
                            if ( event.isPopupTrigger() ){      // for Windows
                                jPopupMenu.show( event.getComponent(), event.getX(), event.getY() );
                            }
                        }
                    };// MouseAdapter subclass

           private final LinkedList     mailStores      = new LinkedList();
//                 final DefaultTreeModel treeModel = new DefaultTreeModel( rootNode );
//                 final JTree            jTree     = new JTree( treeModel );
//                 final JSplitPane       jSplitPane
//                 = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT, true,
//                                   new JScrollPane( jTree ), null );

    //  End of order-critical declarations  
                   final JTabbedPane              jTabbedPane  = new JTabbedPane();
                   final Logger                   log          = Log.get( this.getClass() );
                   final LogPane                  handler      = new LogPane( new JTextArea() );
                   final JPopupMenu               jPopupMenu   = new JPopupMenu();
                   final Container                mainWindow;
                   final Preferences              preferences;
                   final AbstractList <Component> uiComponents = new ArrayList <Component>();
//                 final MailFonts                fonts        = new MailFonts();
           private       Ft___Base                radio        = null;

    //----------------------------------------------------------------------------------------------
    /** Initialize this display window.
     *
     *  @param  mainWindow      the main window Container.
     */
    FtDisplay( Container mainWindow ){
        JScrollPane jScrollPane = new JScrollPane( handler.pane );

        this.mainWindow = mainWindow;
        log.addHandler( handler );
        preferences = Preferences.userNodeForPackage( this.getClass() );
        uiComponents.add( mainWindow );
        ((RootPaneContainer) mainWindow).getContentPane().add( jTabbedPane );

        handler.pane.setLineWrap( true );
        handler.pane.setWrapStyleWord( true );
        handler.pane.addKeyListener( keyAdapter );
        handler.pane.addMouseListener( mouseAdapter );
        jTabbedPane.addTab( "Log", null, jScrollPane, "Activity log" );
        keyStrokeMap.put( KeyStroke.getKeyStroke( KeyEvent.VK_L, InputEvent.ALT_MASK ),
                          jScrollPane );
        log.info( "Starting " + PROGRAM_ID );
        log.info( "Using Java Runtime Environment (JRE) from:  "
                  + System.getProperty( "java.vendor" )
                  + ";  version:  "
                  + System.getProperty( "java.version" ) );
    }// FtDisplay constructor

    //----------------------------------------------------------------------------------------------
    /** Program settings initialization.  Sets up program options.
     *
     */
    void setup(){
        final FtDisplay ftDisplay = this;
              String    value;

        // Setup display
        initMenus();

        Tools.logDiag( log, Level.CONFIG, null, preferences );
    }// FtDisplay.setup

    //----------------------------------------------------------------------------------------------
    /** Performs final initialization.
     *
     *  @throws IOException                         on an I/O error.
     *  @throws PortInUseException                  if the port is in use.
     *  @throws UnsupportedCommOperationException   on a software error.
     *  @throws TooManyListenersException           on a software error.
     */
    void start()                                        throws  IOException,
                                                                PortInUseException,
                                                                UnsupportedCommOperationException,
                                                                TooManyListenersException {

        JScrollPane jScrollPane;
        int index = 0;
        radio = getRadio( "COM1" );
        for ( FtGroup group : radio.getGroups() ){
            jScrollPane = new JScrollPane( new JTable( new FtChannelGroup( group, radio ) ) );
            jTabbedPane.insertTab( group.toString(), null, jScrollPane, group.toString(), index++ );
        }
        jScrollPane = new JScrollPane( new JTable( radio.getOptions() ) );
        jTabbedPane.insertTab( "Settings", null, jScrollPane, "Settings", index++ );
        jScrollPane = new JScrollPane( new JTable( radio.getCalibration() ) );
        jTabbedPane.insertTab( "Calibration", null, jScrollPane, "Calibration", index++ );
    }// FtDisplay.start

    //----------------------------------------------------------------------------------------------
    /** Initializes the pop-up menus.
     */
    void initMenus(){
        JMenu                jMenu;
        JMenuItem            jMenuItem          = null;
        JRadioButtonMenuItem button;
        ButtonGroup          buttonGroup;
        String               value,
                             currLnfName        = UIManager.getLookAndFeel().getName();

        uiComponents.add( jPopupMenu );

        // Set up File submenu
        jMenu = new JMenu( "File" );

        jMenuItem = new JMenuItem( "Load" );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    try {
                        JFileChooser jFileChooser = new JFileChooser( "/" );
                        switch ( jFileChooser.showOpenDialog( mainWindow ) ){
                        case JFileChooser.APPROVE_OPTION:
                            File file = jFileChooser.getSelectedFile();
                            if ( file == null  ||  ! file.exists() ){
                                break;
                            }
                            InputStream inputStream = new FileInputStream( file );
                            load( inputStream );
                            inputStream.close();
                            break;
                        }
                    } catch ( Exception exception ){
                        log.log( Level.WARNING, "Error loading radio settings", exception );
                    }
                }
            } );
        jMenu.add( jMenuItem );

        jMenuItem = new JMenuItem( "Save" );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    try {
                        JFileChooser jFileChooser = new JFileChooser( "/" );
                        switch ( jFileChooser.showOpenDialog( mainWindow ) ){
                        case JFileChooser.APPROVE_OPTION:
                            File file = jFileChooser.getSelectedFile();
                            if ( file == null ){
                                break;
                            }
                            PrintStream printStream = new PrintStream( file );
                            save( printStream );
                            printStream.close();
                            break;
                        }
                    } catch ( Exception exception ){
                        log.log( Level.WARNING, "Error saving radio settings", exception );
                    }
                }
            } );
        jMenu.add( jMenuItem );
        jPopupMenu.add( jMenu );

        // Set up Log Options submenu
        String currentName = handler.getLevel().getName();
        value = preferences.get( "Log.level", currentName );
        jMenu = new JMenu( "Logging level" );
        buttonGroup = new ButtonGroup();
        jMenuItem = null;
        for ( final Level level : LogPane.levels ){
            final String name = level.getName();
            button = new JRadioButtonMenuItem( Tools.initialCase( name ),
                                               name.equals( currentName ) );
            button.setToolTipText( "Set the " + name + " level of activity logging" );
            button.addActionListener( new ActionListener(){
                    public void actionPerformed( ActionEvent event ){
                        handler.setLevel( level );
                        preferences.put( "Log.level", name );
                        log.info( "Setting: The " + name + " level of activity logging" );
                    }
                } ); // ActionListener implementation
//          keyStrokeMap.put( KeyStroke.getKeyStroke( logLevel.charAt( 0 ) ), button );
            buttonGroup.add( button );
            jMenu.add( button );
            if ( name.equalsIgnoreCase( value ) ){
                jMenuItem = button;
            }
        }
        if ( jMenuItem != null ){
            jMenuItem.doClick();
        } else {
            log.warning( "'" + value + "' initial logging level not found, using default" );
        }
        jMenuItem = new JMenuItem( "Clear log" );
        jMenuItem.setToolTipText( "Clear the activity log" );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    handler.clear();
                }
            } ); // ActionListener implementation
        jMenu.add( jMenuItem );
        jPopupMenu.add( jMenu );

        // Set up Look & Feel submenu
        value = preferences.get( "Initial.lookAndFeel", "Native" );
        jMenu = new JMenu( "Look & Feel" );
        buttonGroup = new ButtonGroup();
        jMenuItem = null;
        for ( UIManager.LookAndFeelInfo lookAndFeelInfo : Tools.lookAndFeelInfos ){
            final String className = lookAndFeelInfo.getClassName();
                  String name      = lookAndFeelInfo.getName();
                  int    key       = Character.toUpperCase( name.charAt( 0 ) );
            button = new JRadioButtonMenuItem( name, name.equals( currLnfName ) );
            button.setToolTipText( "Set the Look & Feel to " + name );
            button.addActionListener( new ActionListener(){
                    public void actionPerformed( ActionEvent event ){
                        setLookAndFeel( className );
                    }
                } ); // ActionListener implementation
            keyStrokeMap.put( KeyStroke.getKeyStroke( key,
                                                      InputEvent.CTRL_MASK
                                                      | InputEvent.ALT_MASK ),
                              button );
            buttonGroup.add( button );
            jMenu.add( button );
            if ( value.equalsIgnoreCase( name )
                    || value.equalsIgnoreCase( "Native" )
                    &&  className.equals( UIManager.getSystemLookAndFeelClassName() ) ){
                jMenuItem = button;
            }
        }
        jPopupMenu.add( jMenu );
        if ( jMenuItem != null ){
            jMenuItem.doClick();
        } else {
            log.warning( "'" + value + "' initial look and feel not found, using default" );
        }

        // Set up About menu item
        jMenuItem = new JMenuItem( "About", KeyEvent.VK_A );
        jMenuItem.setToolTipText( "Display information about this program" );
        jMenuItem.addActionListener( new ActionListener(){
                public void actionPerformed( ActionEvent event ){
                    Tools.showDialog( mainWindow, "About " + PROGRAM_ID,
                                      new JOptionPane( COPYRIGHT,
                                                       JOptionPane.INFORMATION_MESSAGE ) );
                }
            } ); // ActionListener implementation
        keyStrokeMap.put( KeyStroke.getKeyStroke( KeyEvent.VK_A, 0 ), jMenuItem );
        jPopupMenu.add( jMenuItem );

        // Set up Exit menu item
        if ( ! (mainWindow instanceof JApplet) ){
            jMenuItem = new JMenuItem( "Exit", KeyEvent.VK_X );
            jMenuItem.setToolTipText( "Exit this program" );
            jMenuItem.addActionListener( new ActionListener() {
                    public void actionPerformed( ActionEvent event ){
                        FtRemote.exit( mainWindow );
                    }
                } ); // ActionListener implementation
            keyStrokeMap.put( KeyStroke.getKeyStroke( KeyEvent.VK_X, 0 ), jMenuItem );
            jPopupMenu.add( jMenuItem );
        }

        keyStrokeMap.put( KeyStroke.getKeyStroke( KeyEvent.VK_M, 0 ), jPopupMenu );
    }// FtDisplay.initMenus

    //----------------------------------------------------------------------------------------------
    /** Sets the look and feel.
     *
     *  @param  className               the look and feel class name.
     */
    void setLookAndFeel( String className ){
        try {
            UIManager.setLookAndFeel( className );
            log.info( "Setting: " + UIManager.getLookAndFeel().getDescription() );
            for ( Component component : uiComponents ) {
                SwingUtilities.updateComponentTreeUI( component );
            }
            PopupFactory.setSharedInstance(new PopupFactory()); 
//          jTree.setRowHeight( Math.max( jTree.getFontMetrics( fonts.base.font ).getHeight(),
//                                        MailTreeCellRenderer.leafIcon.getIconHeight() ) );
        } catch ( Exception exception ){  // multiple
            log.log( Level.WARNING, "Error setting the Look & Feel", exception );
        }
    }// setLookAndFeel.initMenus

    //----------------------------------------------------------------------------------------------
    /** Sets an appropriate title.
     *
     *  @param  title                   a title string.
     */
    void updateTitle( String title ){
        title = "FtRemote " + title;
        if        ( mainWindow instanceof JFrame ){
            ((JFrame) mainWindow).setTitle( title );
        } else if ( mainWindow instanceof JApplet ){            // assert
            ((JApplet) mainWindow).showStatus( title );
        }
    }// FtDisplay.updateTitle

    //----------------------------------------------------------------------------------------------
    /** getRadio connects to the radio and determines a radio of the proper type.
     *
     *  @param  portName                            the COM port name to be used.
     *  @return Ft___Base                              - the radio.
     *  @throws IOException                         on an I/O error.
     *  @throws PortInUseException                  if the port is in use.
     *  @throws UnsupportedCommOperationException   on a software error.
     *  @throws TooManyListenersException           on a software error.
     */
    private Ft___Base getRadio( String portName )          throws  IOException,
                                                                PortInUseException,
                                                                UnsupportedCommOperationException,
                                                                TooManyListenersException {
        TreeMap <String, CommPortIdentifier> map = new TreeMap <String, CommPortIdentifier> ();
        for ( Enumeration enumeration = CommPortIdentifier.getPortIdentifiers();
              enumeration.hasMoreElements(); ){
            CommPortIdentifier commPortId = (CommPortIdentifier) enumeration.nextElement();
            if ( commPortId.getPortType() == CommPortIdentifier.PORT_SERIAL ){
                if ( commPortId.getName().equalsIgnoreCase( portName ) ){
                    log.info( "Loading radio values ..." );
                    SerialIO serialIO = new SerialIO( commPortId, 38400, 100 );
                    byte[]   memory   = serialIO.getMemory();
                    for ( FtRemote.Radios radio : FtRemote.Radios.values() ){
                        if ( radio.memorySize == memory.length ){
                            log.info( "Radio is: " + radio );
                            return radio.getRadio( serialIO, memory );
                        }
                    }
                    throw new IOException( "Unknown radio type [" + memory.length
                                         + "] at port " + portName );
                }
                map.put( commPortId.getName(), commPortId );
            }
        }
        String string = "Port " + portName + " not found;  available ports are:";
        for ( CommPortIdentifier commPortId : map.values() ){
            string += "  " + commPortId.getName();
        }
        throw new IOException( string );
    } // FtDisplay.getRadio

    //----------------------------------------------------------------------------------------------
    /** save writes the radio's settings to a file.
     *
     *  @param  printStream                     the PrintStream of the file to be written.
     *  @throws Exception                       on any error.
     */
    private void save( PrintStream printStream )        throws  Exception {
        Ft___Calibration ftCalibration = radio.getCalibration();
        Ft___Options     ftOptions     = radio.getOptions();

        log.info( "Saving radio values ..." );
        for( FtField field : ftCalibration.getFields() ){
            String name = field.toString().substring( 1 );
            if ( LIST_UNKNOWNS  ||  name.charAt( 0 ) != '_' ){
                printStream.println( "Calibration," + name + "=" + ftCalibration.get( field ) );
            }
        }
        printStream.println();
        for( FtField field : ftOptions.getFields() ){
            String name = field.toString().substring( 1 );
            if ( LIST_UNKNOWNS  ||  name.charAt( 0 ) != '_' ){
                printStream.println( "Option," + name + "=" + ftOptions.get( field ) );
            }
        }
        printStream.println();
        for ( FtGroup group : radio.getGroups() ){
            Object[] keys  = group.getKeys();
            int      index = 0;
            for ( Ft___Channel ftChannel : radio.getChannels( group ) ){
                printStream.print( group.toString()
                                 + (group.getCount() > 1 ? keys != null ? keys[ index ]
                                                                        : "_" + (index + 1)
                                                         : "") );
                for( FtField field : ftChannel.getFields() ){
                    if ( LIST_UNKNOWNS  ||  field.toString().charAt( 0 ) != '_' ){
                        printStream.print( "," + field + "=" + ftChannel.get( field ) );
                    }
                    if ( ftChannel.get( field ) == Ft___Channel.States.UNUSED ){  // no casts !!!
                        break;
                    }
                }
                printStream.println();
                index++;
            }
            printStream.println();
        }
        log.fine( "Saving complete" );
    } // FtDisplay.save

    //----------------------------------------------------------------------------------------------
    /** load reads the radio's settings from a file and updates the radio.
     *
     *  @param  inputStream                     the InputStream of the file to be read.
     *  @throws Exception                       on any error.
     */
    private void load( InputStream inputStream )        throws  Exception {
        BufferedReader                 reader        = new BufferedReader(
                                                           new InputStreamReader( inputStream ) );
        Ft___Calibration                  ftCalibration = radio.getCalibration();
        Ft___Options                      ftOptions     = radio.getOptions();
        String                            line          = "";
        HashMap <FtGroup, Ft___Channel[]> map           = new HashMap <FtGroup, Ft___Channel[]>();

        for ( FtGroup group : radio.getGroups() ){
            map.put( group, radio.getChannels( group ) );
        }
        log.info( "Processing updates ..." );
        while ( line != null ){
            line = reader.readLine();
            if ( line == null  ||  ".".equals( line ) ){
//              log.fine( radio.setCalibration( ftCalibration )
//                      + " bytes updated in calibration group." );
                log.fine( radio.setOptions( ftOptions ) + " bytes updated in option group." );
                for ( FtGroup group : radio.getGroups() ){
                    Ft___Channel[] ftChannels = map.get( group );
                    log.fine( radio.setChannels( group, ftChannels ) + " bytes updated in "
                            + group + " channel group (" + ftChannels.length + ")." );
                }
            } else if ( line.length() > 0 ){
                StringTokenizer tokens = new StringTokenizer( line );
                String          token  = tokens.nextToken( "_," );
                if ( token.equals( "Calibration" ) ){
                    token = tokens.nextToken( "=" );
                    FtField field = ftCalibration.getField( "_" + token.substring( 1 ) );
                    ftCalibration.put( field, tokens.nextToken( "" ).substring( 1 ) );
                } else if ( token.equals( "Option" ) ){
                    token = tokens.nextToken( "=" );
                    FtField field = ftOptions.getField( "_" + token.substring( 1 ) );
                    ftOptions.put( field, tokens.nextToken( "" ).substring( 1 ) );
                } else {
                    FtGroup     group      = radio.getGroup( token );
                    Ft___Channel[] ftChannels = map.get( group );
                    int index = 0;
                    if ( group.getCount() > 1 ){
                        Object[] keys = group.getKeys();
                        token = tokens.nextToken( "," );
                        if ( token.equals( ".*" ) ){
                            token = tokens.nextToken( "" );
                            for ( Ft___Channel ftChannel : ftChannels ){
                                putChannel( ftChannel, new StringTokenizer( token ) );
                            }
                            break;
                        } else if ( keys != null ){
                            index = FtRemote.getEnum( keys, token ).ordinal();
                        } else {
                            index = Integer.parseInt( token.substring( 1 ) ) - 1;
                        }
                    }
                    putChannel( ftChannels[ index ], tokens );
                }
            }
        }
        log.fine( "Updating complete" );
    } // FtDisplay.load

    //----------------------------------------------------------------------------------------------
    /** putChannel updates a specific channel's data.
     *
     *  @param  ftChannel                       the Ft___Channel to be updated.
     *  @param  tokens                          the StringTokenizer containing the update values.
     */
    private void putChannel( Ft___Channel ftChannel, StringTokenizer tokens ){
        while ( tokens.hasMoreTokens() ){
            String token = tokens.nextToken( "=" );
            FtField field = ftChannel.getField( token.substring( 1 ) );
            token = tokens.nextToken( field.type() == FtField.Type.TAG ? "" : "," );
            ftChannel.put( field, token.substring( 1 ) );
        }
    } // FtDisplay.putChannel

}// FtDisplay class
